
# In the following we have 59 different procedures for around 2700 lines.
# 
# Maple implementations of the algorithms involved in my Ph.D. thesis: Algorithmic Computation of Formal Fourier Series.
# The code are divided in 4 parts: Part 1 is devoted to some preliminaries. Part 2 is devoted to the computation of Fourier coefficients: This part is divided in two sections: Section 1  is devoted to algorithms involving differential equations with coefficients in K[cos(wt), sin(wt)] whereas Section 2 is for those involving differential equations with coefficients in K[e^{-iwt}, e^{iwt}]. Part 3  for the implementation not depending on the form of the differential equation. In part 4 we stored some procedure used in some previous one.

# Part1. Preliminaries.
# 
# Since Maple library does not provide us some membership test, for example for trigonometric polynomial, rational trigonometric functions, we first implement them. And then we implement also a membership test for trigonometric holonomic differential equations. 
# 
# We first recall that TH(w) is the set of functions satisfying linear differential equations with coefficients in K[cos(wt), sin(wt)]
# 1-1 Membership test for trigonometric polynomials.
# 
# The following procedure checks if a given function is a trigonometric polynomial and returns true or false. Its uses other implementations which are in stored in part4.

`type/TRIGPOLY`:=proc(f,t)
local g,b,c,i,d,A,B,j,e,num,den,s,r,w,v,h,R,NUM,RR,l,y,z,u,C;
g:=convert(f,trig);
g:=normal(expand(g));
num:=numer(g);
den:=denom(g);
# check if the input is a constant or a polynomial function.  
if type(g,polynom(anything,t)) then
    if has(g,t) then 
    return false;
    else return true;    
    fi;
fi;
#check if the input function has a denominator containing the variable t
if has(den,t) then
return false;
fi;
#check if the input is a constant 
if not has(num,t) and not has(den,t) then
return true; 
fi;
#We collect here the term containing sqrt, wenn there exist
NUM:=num;
R:={};
if nops({NUM})=1 and type(NUM,whattype(2*t)) then 
    R:={op(R),(op(NUM))};
else 
    R:={NUM};
fi;
for i from 1 to nops(NUM) do
    if nops(op(i,NUM))=1 then 
        R:={op(R),op(i,NUM)}; 
    else R:={op(R),op(op(i,NUM))};
    fi;
od;
R:=select(type,R,`^`);
RR:={};
   for i from 1 to nops(R) do
       if not has(op(i,R),t) then 
            RR:={op(RR),op(i,R)};         
       fi;
   od; 
R:= R minus RR;
for i from 1 to nops(R) do
    if type(op(2,op(i,R)),fraction) then 
        return false;         
    fi;
od;  
g:=select(has,num,t);
if nops(g)=1 then 
    g:=expand(g);
    if not nops(g)=1 then
        g:={seq(select(has,op(i,(g)),t),i=1..nops(g))};
        g:=[seq(select(has,op(i,(g)),t),i=1..nops(g))];
        #g:={select(has,op(g),t)};
        if nops(g)=1 then 
            if not has(op(op(1,g))/t,t)  then 
                return true;
            fi;
                return  false;
        fi;
        easy(g,t);
        else
        if type(g,`^`) then 
           return false;
        fi; 
        if not has(op(g)/t,t) then 
            return true;
        else 
            return false;
        fi
    fi;
fi;
c:=select(has,g,t); #To put down the power for example from  #(cos(t)+sin(a)^2)^9 to (cos(t)+sin(a)^2)
c:={g};
d:={};
i:=1;
 while  i < (nops(c)+1) and type(op(i,c),whattype(2*t)) do
d:={op(d),op(op(i,c))}
;i:=i+1;
g:=(d);
od;
g:={seq(select(has,op(i,(g)),t),i=1..nops(g))};
if nops(g)=1 then  
    g:=select(has,op(g),t); #To put down the powers, for example from cos(t)^4 to cos(t) 
    if not type(g,trig(t)) then 
        if has(g,t) then 
            return false;
        fi;
           return  true; 
    fi;
    g:=expand(g);
    if not nops(g)=1 then
        g:={seq(select(has,op(i,(g)),t),i=1..nops(g))};
        g:=[seq(select(has,op(i,(g)),t),i=1..nops(g))];
        #g:={select(has,op(g),t)};
        if nops(g)=1 then 
            if not has(op(op(1,g))/t,t)  then 
                return true;
            fi;
                return  false;
        fi;
        easy(g,t);
        else
        if type(g,`^`) then 
           return false;
        fi; 
        if not has(op(g)/t,t) then 
            return true;
        else 
            return false;
        fi
    fi;
fi;
d:={};
for i from 1 to nops(g) do  
    if type(op(i,g),whattype(2*t)) then 
        d:={op(d),op(op(i,g))} 
    else
        d:={op(d),op(i,g)};
    fi;
od;
b:=select(has,d,t);
w:={};
s:=1;
 while  s < (nops(b)+1) do
    if nops(op(s,b))=1 then
    w:={op(w),op(s,b)};
    s:=s+1;
    else
    w:={op(w),op(op(s,b))};
    s:=s+1;
    fi;     
od;
b:=w;
b:={seq(select(has,op(i,(b)),t),i=1..nops(b))};
b:={seq(select(has,op(i,(b)),t),i=1..nops(b))};#To avoid cases of the form {undefined, .....}
#b:=select(has,b,t);
if nops(b)=1 then
    if not nops(op(b))=1 then
        e:={};
        for i from 1 to nops(op(b)) do  
            if type(op(i,op(b)),whattype(2*t)) then 
                e:={op(e),op(op(i,op(b)))} 
            else
            e:={op(e),op(i,op(b))};
            fi;
        od;
        b:=select(has,e,t);
        for i from 1 to nops(b) do
            if not type(op(i,b),trig) then 
            return false;
            fi;
        od;
    else
        if not type(op(1,b),trig(t)) then
            return false;
        fi;
        if  has(op(1,op(1,b))/t,t) then 
        return false;
        fi;   
        return true;  
    fi;
fi;

l:={};
for z from 1 to nops(b) do 
    if type(op(z,b),whattype(2*t)) then
        l:={op(l),op(op(z,b))};
    else l:={op(l),op(z,b)};
    fi;    
od;
b:=l;
b:={seq(select(has,op(i,(b)),t),i=1..nops(b))};


for i from 1 to nops(b) do
    if not type(op(i,b),trig) then 
        return false;
    fi;
od;
b:=expand(b);
v:={};
for i from 1 to nops(b) do
    if nops(op(i,b))=1 then 
        v:={op(v),op(i,b)}; 
    else v:={op(v),op(op(i,b))};
    fi;
od;
b:={seq(select(has,op(i,(v)),t),i=1..nops(v))};
b:=[seq(select(has,op(i,(b)),t),i=1..nops(b))];
if nops(b)=1 then
    if not has(op(b)/t,t) then 
        return true;
    fi;
      return false;
fi;
easytrig(b,t);
end proc: #end `type/TRIGPOLY`(199)


# The following procedure checks  if a given function is a trigonometric polynomial. In the positive case its returns a number $w$ for which the given function is in TH(w).   Its uses other implementations which are in stored in part4.

TRIGPOLYtest:=proc(f,t)
local g,b,c,i,d,A,B,j,e,num,den,s,r,w,v,h,R,NUM,RR,l,y,u,z,C;
g:=convert(f,trig);
g:=normal(expand(g));
num:=numer(g);
den:=denom(g);
if type(g,polynom(anything,t)) then
    if has(g,t) then 
    return false;
    else return 10;    
    fi;
fi;
if has(den,t) then
return false;
fi;
if not has(num,t) and not has(den,t) then
return 10;
fi;
NUM:=num;
R:={};
if nops({NUM})=1 and type(NUM,whattype(2*t)) then 
    R:={op(R),(op(NUM))};
else 
    R:={NUM};
fi;
for i from 1 to nops(NUM) do
    if nops(op(i,NUM))=1 then 
        R:={op(R),op(i,NUM)}; 
    else R:={op(R),op(op(i,NUM))};
    fi;
od;
R:=select(type,R,`^`);
RR:={};
for i from 1 to nops(R) do
    if not has(op(i,R),t) then 
        RR:={op(RR),op(i,R)};         
    fi;
od; 
R:= R minus RR;
for i from 1 to nops(R) do
    if type(op(2,op(i,R)),fraction) then 
        return false;         
    fi;
od;  
g:=select(has,num,t);   
if nops(g)=1 then 
    g:=expand(g);
    if not nops(g)=1 then
        g:={seq(select(has,op(i,(g)),t),i=1..nops(g))};
        g:=[seq(select(has,op(i,(g)),t),i=1..nops(g))];
        #g:={select(has,op(g),t)};
        if nops(g)=1 then 
            if not has(op(op(1,g))/t,t)  then 
                return (op(op(1,g))/t);
            fi;
                return  false;
        fi;
        easy(g,t);
    else
        if type(g,`^`) then 
           return false;
        fi; 
        if not has(op(g)/t,t) then 
            return (op(g)/t);
        else 
            return false;
        fi
    fi;
fi;
c:=select(has,g,t); ##  To put down the power for example from  
#(cos(t)+sin(a)^2)^9 to (cos(t)+sin(a)^2)
c:={c}; 
d:={};
i:=1;
while  i < (nops(c)+1) and type(op(i,c),whattype(2*t)) do
d:={op(d),op(op(i,c))};
i:=i+1;
g:=(d);
od;
g:={seq(select(has,op(i,(g)),t),i=1..nops(g))};
if nops(g)=1 then
    g:= select(has,op(g),t); ## To put down the powers, for example from cos(t)^4 to cos(t)
 
    if not type(g,trig(t)) then 
        if has(g,t) then 
            return false;
        fi;
            return  10; 
    fi;
g:=expand(g);
    if not nops(g)=1 then
        g:={seq(select(has,op(i,(g)),t),i=1..nops(g))};
        g:=[seq(select(has,op(i,(g)),t),i=1..nops(g))];
        #g:={select(has,op(g),t)};
        if nops(g)=1 then 
            if not has(op(op(1,g))/t,t)  then 
                return (op(op(1,g))/t);
            fi;
                return  false;
        fi;
        easy(g,t);
    else
        if type(g,`^`) then 
           return false;
        fi; 
        if not has(op(g)/t,t) then 
            return (op(g)/t);
        else 
            return false;
        fi
    fi;
fi;
d:={};
for i from 1 to nops(g) do  
    if type(op(i,g),whattype(2*t)) then 
        d:={op(d),op(op(i,g))} 
    else
        d:={op(d),op(i,g)};
    fi;
od;
b:=select(has,d,t);
w:={};
s:=1;
while  s < (nops(b)+1) do
    if nops(op(s,b))=1 then
        w:={op(w),op(s,b)};
        s:=s+1;
    else
        w:={op(w),op(op(s,b))};
        s:=s+1;
    fi;     
od;
b:=w;
b:={seq(select(has,op(i,(b)),t),i=1..nops(b))};
b:={seq(select(has,op(i,(b)),t),i=1..nops(b))};## To avoid cases of the form {undefined, .....}

#b:=select(has,b,t);
if nops(b)=1 then
    if not nops(op(b))=1 then
        e:={};
        for i from 1 to nops(op(b)) do  
            if type(op(i,op(b)),whattype(2*t)) then 
                e:={op(e),op(op(i,op(b)))} 
            else
                e:={op(e),op(i,op(b))};
            fi;
        od;
        b:=select(has,e,t);
        for i from 1 to nops(b) do
            if not type(op(i,b),trig) then 
                return false;
            fi;
        od;
    else
        if not type(op(1,b),trig(t)) then
            return false;
        fi;
        if  has(op(1,op(1,b))/t,t) then 
            return false;
        fi;  
        return (op(1,op(1,b))/t);   
    fi;
fi;
l:={};
for z from 1 to nops(b) do 
    if type(op(z,b),whattype(2*t)) then
        l:={op(l),op(op(z,b))};
    else l:={op(l),op(z,b)};
    fi;    
od;
b:=l;
b:={seq(select(has,op(i,(b)),t),i=1..nops(b))};
for i from 1 to nops(b) do
    if not type(op(i,b),trig) then 
        return false;
    fi;
od;
b:=expand(b);
v:={};
for i from 1 to nops(b) do
    if nops(op(i,b))=1 then 
        v:={op(v),op(i,b)}; 
    else
        v:={op(v),op(op(i,b))};
    fi;
od;
b:={seq(select(has,op(i,(v)),t),i=1..nops(v))};
b:=[seq(select(has,op(i,(b)),t),i=1..nops(b))];
if nops(b)=1 then
    if not has(op(b)/t,t) then 
        return (op(b)/t);
    fi;
      return false;
fi;
easy(b,t);
end proc:#end TRIGPOLYtest(193)

# 1-2 Membership test for rational trigonometric functions.
# 
# The following procedure checks  if a given function is a rational trigonometric functions and returns true or false. 
`type/RATTRIG`:=proc(f,t)
local g,num,den,deno,i,numo,a,b,c,d,e,alpha,beta,A,B,C,m,p,E,w,x,y,q,j;
g:=normal(f);
num:=(numer(f));
den:=(denom(f));
if not has(g,t) then 
    return true;
fi;
if not type(num,TRIGPOLY(t)) then
    return false;
fi;
if not type(den,TRIGPOLY(t)) then
    return false;
fi;
if TRIGPOLYtest(num,t)=10 then 
    return true;
fi;
if TRIGPOLYtest(den,t)=10 then 
    return true;
fi;
if type(TRIGPOLYtest(num,t)/TRIGPOLYtest(den,t),integer) or 
type(TRIGPOLYtest(num,t)/TRIGPOLYtest(den,t),rational) then
A:=[TRIGPOLYtest(num,t),TRIGPOLYtest(den,t)];
B:=[seq(denom(op(z,A)),z=1..nops(A))];
C:=[seq(numer(op(u,A)/t),u=1..nops(A))];
m:={};
    for p from 1 to nops((C)) do  
        if not type(op(p,(C)),whattype(2*t)) then 
            m:={op(m),(op(p,C))} ;
        else
            m:={op(m),op(op(p,C))};
        fi;
    od;
w:=m;
x:={};
y:={};
for q from 1 to nops(w) do ;
    if type(op(q,w),whattype(sqrt(2))) then  
        x:={op(x),op(q,w)}; 
    else
        y:={op(y),op(q,w)};
    fi;
od;  
if x={} then
    for i from 1 to nops(C) do      
        for j from i+1 to nops(C) do
            if not op(i,C)=op(j,C) then
                E:=[op(1,C),op(2,C)];
                return true;
            fi;
        od; 
    od; 
fi;
if nops(x)=1 then 
    x:=op(x);
else
    x:=product(op(o,x),o=1..nops(x));
fi;
C:=[seq(op(i,C)/x,i=1..nops(C))];
return true;
fi;
return false;
end proc:# end `type/RATTRIG` (63)

# The following procedure checks  if a given function is a rational trigonometric function. In the positive case its returns a number $w$ for which the given function is in 
#  TH(w).   
RATTRIGtest:=proc(f,t)
local g,num,den,deno,i,numo,a,b,c,d,e,alpha,beta,A,B,C,m,p,E,w,x,y,q,j;
g:=normal(f);
num:=(numer(f));
den:=(denom(f));
if not has(g,t) then 
    return w;
fi;
if not type(num,TRIGPOLY(t)) then
    return false;
fi;
if not type(den,TRIGPOLY(t)) then
    return false;
fi;
if TRIGPOLYtest(num,t)=10 then 
    return TRIGPOLYtest(den,t);
fi;
if TRIGPOLYtest(den,t)=10 then 
    return TRIGPOLYtest(num,t);
fi;
if type(TRIGPOLYtest(num,t)/TRIGPOLYtest(den,t),integer) or 
type(TRIGPOLYtest(num,t)/TRIGPOLYtest(den,t),rational) then
A:=[TRIGPOLYtest(num,t),TRIGPOLYtest(den,t)];
B:=[seq(denom(op(z,A)),z=1..nops(A))];
C:=[seq(numer(op(u,A)/t),u=1..nops(A))];
m:={};
    for p from 1 to nops((C)) do  
        if not type(op(p,(C)),whattype(2*t)) then 
            m:={op(m),(op(p,C))} ;
        else
            m:={op(m),op(op(p,C))};
        fi;
    od;
w:=m;
x:={};
y:={};
for q from 1 to nops(w) do ;
    if type(op(q,w),whattype(sqrt(2))) then  ;
      x:={op(x),op(q,w)}; 
    else
       y:={op(y),op(q,w)};
     fi;
  od;  
if x={} then
for i from 1 to nops(C) do      
    for j from i+1 to nops(C) do
        if not op(i,C)=op(j,C) then
            E:=[op(1,C),op(2,C)];
            return gcd(op(E))/lcm(op(B));
        fi;
     od; 
 od; 
fi;
if nops(x)=1 then 
   x:=op(x);
else
   x:=product(op(o,x),o=1..nops(x));
fi;
C:=[seq(op(i,C)/x,i=1..nops(C))];
return  x*gcd((op(C)))/lcm(op(B));
fi;
return false;
end proc: #end RATTRIGtest (63)

# 1-3 Membership test for trigonometric holonomic differential equations
# 
# The following procedure checks  if a given differential equation is a trigonometric holonomic one and returns true or false.
`type/TH_DE`:=proc(DE,Ft)
local F,t,localde,opde,i,aa,d,A,coef,b,j;
F:=op(0,Ft);
t:=op(1,Ft);
if op(2,DE)=0 then 
localde:=op(1,DE);
else
localde:=DE;
fi;
if not has(localde,F(t)) then 
return false;
fi;
opde:=[op(localde)];
opde:=select(has,opde,F);
for i from 1 to nops(opde) do
    if type(op(i,opde),whattype(sqrt(2))) then
        return false;
    fi;
od;
for i from 1 to nops(opde) do
    aa:={op(op(i,opde))};
    aa:=select(has,aa,F);
    if nops(aa)=1 and type(op(aa),whattype(sqrt(2))) then 
        return false;
    fi;
    if not (nops(aa)=0 or  nops(aa)=1) then 
        return false;
    fi;
od;
d:=PDEtools[difforder](localde,t);
for i from d to 0 by -1  do
    localde:= subs(diff(F(t),[t$i])=X^i,localde);
    coef:=[seq(coeff(localde,X,i),i=0..d)];
od;
coef:=select(has,coef,t);
A:=[];
for i from 1 to nops(coef) do
    if not type(coef[i],TRIGPOLY(t)) then 
        return  false;
    else
        A:=[op(A),TRIGPOLYtest(coef[i],t)];
    fi;
od;

if nops(A)=0 then 
    return true;
fi;
if nops(A)=1 then 
    return true;
fi;
for i from 1 to nops(coef) do
    for j from i+1 to nops(coef) do
        if not type(TRIGPOLYtest(coef[i],t)/TRIGPOLYtest(coef[j],t),integer) and  not type(TRIGPOLYtest(coef[i],t)/TRIGPOLYtest(coef[j],t),fraction)  then 
            return false;
        fi;
    od;
od;
for i from 1 to nops(A) do
    for j from i+1 to nops(A) do
        if not type(op(i,A)/op(j,A),integer)  and not type(op(i,A)/op(j,A),fraction) then 
            return false;
        fi;
        return true;
    od;
od;
end proc:#end `type/TH_DE`(66)

# The following procedure checks  if a given differential equation DE is a trigonometric holonomic one and returns a number $w$ for which DE is in TH(w) ,in the positive case, and returns some justifications in the negative case.
THDEtest:=proc(DE,Ft)
local F,t,localde,opde,i,aa,d,A,coef,b,j;
F:=op(0,Ft);
t:=op(1,Ft);
if op(2,DE)=0 then 
localde:=op(1,DE);
else
localde:=DE;
fi;
if not has(localde,F(t)) then 
return (DE,"is not a function of",F(t));
fi;
opde:=[op(localde)];
opde:=select(has,opde,F);
for i from 1 to nops(opde) do
    if type(op(i,opde),whattype(sqrt(2))) then
        return ("the differential equation ",DE,"
             is not linear");
    fi;
od;
for i from 1 to nops(opde) do
    aa:={op(op(i,opde))};
    aa:=select(has,aa,F);
    if nops(aa)=1 and type(op(aa),whattype(sqrt(2))) then 
        return ("the differential equation ",DE,"
             is not linear");
    fi;
    if not (nops(aa)=0 or  nops(aa)=1) then 
        return ("the differential equation ",DE,"
             is not linear");
    fi;
od;
d:=PDEtools[difforder](localde,t);
for i from d to 0 by -1  do
    localde:= subs(diff(F(t),[t$i])=X^i,localde);
    coef:=[seq(coeff(localde,X,i),i=0..d)];
od;
coef:=select(has,coef,t);
A:=[];



for i from 1 to nops(coef) do
    if not type(coef[i],TRIGPOLY(t)) then 
        return  (DE,"is not a trigonometric holonomic differential equation, \n since",coef[i],"is not a trigonometric polynomial");
    else
        A:=[op(A),TRIGPOLYtest(coef[i],t)];
    fi;
od;

if nops(A)=0 then 
    return w;
fi;
    
if nops(A)=1 then return
    op(A);
fi;
for i from 1 to nops(coef) do
    for j from i+1 to nops(coef) do
        if not type(TRIGPOLYtest(coef[i],t)/TRIGPOLYtest(coef[j],t),integer) and  not type(TRIGPOLYtest(coef[i],t)/TRIGPOLYtest(coef[j],t),fraction)  then 
            return (DE,"is not a trigonometric holonomic differential equation, \n since the periods of its coefficients",coef[i],"and",coef[j],"are not commesurable");
        fi;
    od;
od;
for i from 1 to nops(A) do
    for j from i+1 to nops(A) do
        if not type(op(i,A)/op(j,A),integer)  and not type(op(i,A)/op(j,A),fraction) then 
            return false;
        fi;
            return op(1,A);
    od;
od;
end proc: #end THDEtest(73)

# Part2  Computation of Fourier coefficients.
# Section A:   Procedures involving differential equations with coefficients in K[cos(wt), sin(wt)] 
# 
# 2-A-1 Computation of the trigonometric differential equations.

# 2-A-1-1 Determination of the efficient P and L.

# The following procedure determines the efficient P and L for which one may get a trigonometric holonomic 
# differential equation leading to a recurrence equation of lowest order.
# 
# It first check  if the given input is rational trigonometric function. 
EfficientPL:=proc(f,t)
local g,w,num,den,numlist,numerlist,denomlist,N,M,j,m,L,i,n,alpha,P,l;
g:=convert(f,trig);
## It first checks if the input is a rational trigonometric function.

w:=RATTRIGtest(g,t);
if w=10 then 
    return  true;
fi;
if not w=false then 
    g:=subs(t=t/w,g);
## In the affirmative case it compute the corresponding P and L.

num:=factor(expand(numer(g)));
den:=factor(expand(denom(g)));
numerlist:=convert(num,multiset);
numlist:=[seq(select(has,op(1,op(i,numerlist)),t),i=1..nops(numerlist))];
denomlist:=convert(den,multiset);
denomlist:=[seq(select(has,op(1,op(j,denomlist)),t),j=1..nops(denomlist))];
N:=0;
M:=0;
for j from 1 to nops(denomlist) do
    if op(j,denomlist)=0 then
        m[j]:=0;
        M:=M+m[j];  
    else
        m[j]:=degree(op(j,denomlist));
        M:=M+m[j];
    fi;
od;
L:=M;
for i from 1 to nops(numlist) do 
    if op(i,numlist)=0 then 
        n[i]:=0;
        N:=N+alpha[i]*n[i];
    else
    n[i]:=degree(op(i,numlist));
    alpha[i]:=op(2,op(i,numerlist)); 
    N:=N+alpha[i]*n[i];
    fi;
od; 
P:=1+N;
return ["L"=L,"P"=P]; 
fi;
return false;
end proc:#end EfficientPL (44)

# The following procedure does the same thing as the previous one, but returns also w corresponding to the efficient P and L in TH(w)
Efficient_PL:=proc(f,t)
local g,w,num,den,numlist,numerlist,denomlist,N,M,j,m,L,i,n,alpha,P,l;
g:=convert(f,trig);
## It first checks if the input is a rational trigonometric function.

w:=RATTRIGtest(g,t);
if w=10 then 
    return  true;
fi;
if not w=false then 
    g:=subs(t=t/w,g);
## In the affirmative case it compute the corresponding P and L.

num:=factor(expand(numer(g)));
den:=factor(expand(denom(g)));
numerlist:=convert(num,multiset);
numlist:=[seq(select(has,op(1,op(i,numerlist)),t),i=1..nops(numerlist))];
denomlist:=convert(den,multiset);
denomlist:=[seq(select(has,op(1,op(j,denomlist)),t),j=1..nops(denomlist))];
N:=0;
M:=0;
for j from 1 to nops(denomlist) do
    if op(j,denomlist)=0 then
        m[j]:=0;
        M:=M+m[j];  
    else
        m[j]:=degree(op(j,denomlist));
        M:=M+m[j];
    fi;
od;
L:=M;
for i from 1 to nops(numlist) do 
    if op(i,numlist)=0 then 
        n[i]:=0;
        N:=N+alpha[i]*n[i];
    else
    n[i]:=degree(op(i,numlist));
    alpha[i]:=op(2,op(i,numerlist)); 
    N:=N+alpha[i]*n[i];
    fi;
od; 
P:=1+N;
print("w=",w);
return ["L"=L,"P"=P]; 
fi;
return (f," ist not a rational trigonometric function");
end proc:#end Efficient_PL (45)


# This procedure computes a trigonometric holonomic differential equation in the case of trigonometric polynomials.
# The computation can be done for symbolic w, but the obtained differential equation is not always the one leading to a recurrence equation of lowest order.
TrigPolDe:=proc(f,Ft)
local F,t;
F:=op(0,Ft);
t:=op(1,Ft);
print(f,"satisfies the following trigonometric holonomic differential equation:");
f*diff(F(t),t)-diff(f,t)*F(t)=0;
end proc: #end TrigPolDe (7)
# This procedure computes a trigonometric holonomic differential equation in the case of rational trigonometric functions.
# The computation can be done for symbolic w, but the obtained differential equation is not always the one leading to a recurrence equation of lowest order.
RatTrigDe:=proc(f,Ft)
local g,F,t,h,de;
F:=op(0,Ft);
t:=op(1,Ft);
g:=numer(f);
h:=denom(f);
de:=combine(g*h*diff(F(t),t)-(diff(g,t)*h-g*diff(h,t))*F(t));
print(f,"satisfies the following trigonometric holonomic differential equation:");
collect( numer(normal(de)),diff, factor)=0;
end proc: #end(RatTrigDe(10)

# 2-A-1-2  Procedure for the computation of trigonometric holonomic differential equations of exact given P and L.
# To avoid conflict with some transcendental  functions, we write the following procedure dec, which replace those of them who are constant with respect to the considered variable, by some symbolic values, which will be restitute at the end of the corresponding computation.

THDEPL:=proc(f,Ft,P,L,w)
local num,num1,den,den1,numRule,denRule,g,F,t,alpha,beta,ansatz,ANSATZ,vars,l,p,entry,eqs,EQS,sol,ENTRY,i,j,res;
F:=op(0,Ft);
t:=op(1,Ft);
g:=f;
if not has(g,t) then
    return sin(w*t)*diff(F(t),t)=0;
fi;
num:=numer(g);
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=denom(g);
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
g:=num/den;
g:=convert(g,trig);
g:=combine(normal(g));
#localde:=subs(numRule,denRule,localde);
ansatz:=0; ANSATZ:=0;vars:={};
for l from 0 to L do
    for p from 0  to P do
        ansatz:=ansatz+(alpha[p,l]*cos(l*w*t)+beta[p,l]*sin(l*w*t))*diff(g,[t$p]); 
        ANSATZ:=ANSATZ+(alpha[p,l]*cos(l*w*t)+beta[p,l]*sin(l*w*t))*diff(F(t),[t$p]); 
        vars:={op(vars),alpha[p,l],beta[p,l]}; 
        entry:=combine(numer(normal(ansatz)),trig); 
    end do;
end do;
eqs:={coeffs(entry,indets(entry,function))}; 
EQS:=map(term->coeffs(term,t),eqs); 
sol:=solve(EQS,vars); 
ENTRY:=subs(sol,ANSATZ);
for i from 0 to l do 
    for j from 0 to p do  
        ENTRY:=numer(subs(alpha[j,i]=1,beta[j,i]=1,ENTRY));
    od;
od;  
res:=collect(ENTRY,diff,factor);  
if not res=0 then
    print(subs(numRule,denRule,f),"satisfies the following trigonometric holonomic differential equation:");
      res:=subs(numRule,denRule,res);
    return res=0; 
else 
    Error ;
end if; 
end proc:#endTHDEPL (47)

# 2-A-1-3  Procedure for the computation of trigonometric holonomic differential equations for rational trigonometric functions, taking in consideration the efficient P and L.
RatTHDE:=proc(f,Ft,w)
local pl,L,P,g,F,t,vi,L_Max,P_Max,ansatz,ANSATZ,vars,k,l,p,delocal;
g:=convert(f,trig);
F:=op(0,Ft);
t:=op(1,Ft);
pl:=EfficientPL(subs(t=t/w,g),t);
if pl=false then 
    return (f," ist not a rational trigonometric function");
fi;
L:=op(2,op(1,pl));
#P:=op(2,op(2,pl))-1;
 P:=1;
vi:=5;
L_Max:=L+vi;
P_Max:=P+vi; 
ansatz:=0;
ANSATZ:=0;
vars:={};
delocal:=Error;
for k from 0  
    while delocal=Error and L_Max < 2*vi do
        for l from L to L_Max do
            for p from P_Max -vi  to P_Max do
                delocal:=THDEPL(f,Ft,p,l,w);
                if not  delocal=0 and not delocal=Error then
                    return delocal; 
                else 
                    delocal:=Error;
                fi;                       
            od;
       od;
    L_Max:=L_Max+vi; P_Max:=P_Max+vi; 
print(k) ;
od;
end proc: #endRatTHDE (31)

# 2-A-1-4 General case including the rational trigonometric functions.
# The following procedure does the same thing as the previous, but does not use the procedure THDEPL and does not check whether the giving function 
# is a rational trigonometric function or not.
ThDEE:=proc(f,Ft,w)
local F,t,g,num,num1,numRule,den,den1,denRule,vi,L_Max,P_Max,ansatz,ANSATZ,vars,res,k,l,p,entry,eqs,EQS,sol,ENTRY,i,j;
F:=op(0,Ft);
t:=op(1,Ft);
g:=f;
if not has(g,t) then
    return sin(w*t)*diff(F(t),t)=0;
fi;
num:=numer(g);
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=denom(g);
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
g:=num/den;
g:=convert(g,trig);
vi:=5;
L_Max:=vi;
P_Max:=vi;
ansatz:=0; ANSATZ:=0;vars:={};
res:=Error;
for k from 0 
    while res=Error and L_Max < 2*vi do
        for l from 0 to L_Max do
            for p from P_Max -vi   to P_Max do
                ansatz:=ansatz+(alpha[p,l]*cos(l*w*t)+beta[p,l]*sin(l*w*t))*diff(g,[t$p]); 
                ANSATZ:=ANSATZ+(alpha[p,l]*cos(l*w*t)+beta[p,l]*sin(l*w*t))*diff(F(t),[t$p]); 
                vars:={op(vars),alpha[p,l],beta[p,l]}; 
                entry:=combine(numer(normal(ansatz)),trig); 
                eqs:={coeffs(entry,indets(entry,function))}; 
                EQS:=map(term->coeffs(term,t),eqs); 
                sol:=solve(EQS,vars); 
                ENTRY:=subs(sol,ANSATZ);
                    for i from 0 to l do 
                        for j from 0 to p do  
                            ENTRY:=numer(subs(alpha[j,i]=1,beta[j,i]=1,ENTRY));
                        od;
                    od;  
                res:=collect(ENTRY,diff,factor);  
                    if not res=0 then
                        print(subs(numRule,denRule,f),"satisfies the following trigonometric holonomic  differential equation:");
                        res:=subs(numRule,denRule,res);
                        return res=0; 
                    else 
                        res:=Error
                    fi; 
            od;
        od;
    L_Max:=L_Max+vi;
    P_Max:=P_Max+vi; 
    print(k);
od;
print(k);
res;
end proc:#end ThDEE (57)

# 2-A-1-5 We give here the general code for the computation of a trigonometric holonomic differential equation, satisfied by a trigonometric holonomic function.
# It first test if the input function is a rational trigonometric function and use the procedure RatTHDE if it is the case. Otherwise
# It starts with P=1 and L=0 and go gradually until the result.
# We remark that if the input function is not in TH, the algorithm will be stopped after a reasonable run.
# 
THDE:=proc(g,Ft,w) 
local localde,f, F,t,vi,L_Max,P_Max,ansatz,ANSATZ,vars,res,k,l,p;
F:=op(0,Ft);
t:=op(1,Ft);
f:=g;
if not has(f,t) then
    return sin(w*t)*diff(F(t),t)=0;
fi;
if  type(f,RATTRIG(t)) then
 return(RatTHDE(f,Ft,w));
fi;
vi:=5;
L_Max:=vi;
P_Max:=vi;
ansatz:=0;
ANSATZ:=0;
vars:={};
res:=Error;
for k from 0 
    while res=Error and L_Max < 2*vi do
        for l from 0 to L_Max do
            for p from P_Max -vi +1  to P_Max do
               res:=THDEPL(f,Ft,p,l,w); 
                   if not res= Error then
                       return res; 
                   fi; 
            od;      
        od; 
    L_Max:=L_Max+vi; 
    P_Max:=P_Max+vi; 
    print(L_Max);
od;
print ("There is no trigonometric holonomic differential equation DE satisfied by",f," in a reasonnable time.\n If such DE exists, they should be get for L>",L_Max-vi,"and P>",P_Max-vi);
res;
end proc: #end THDE (35)

# 2-A-2 Computation of recurrence equations.
# 2-A-2-1  Procedure for the determination of the order of a recurrence equation.
orderrecur:=proc(RE,cn)
local ind,i,l,c;
c:=op(0,cn);
l:={};
ind:=select(has,indets(RE),c);
for i from 1 to nops(ind) do
 if has(op(i,ind),c) then
l:={op(l),op(1,op(i,ind))-n};
  fi;
od;
max(op(l))-min(op(l));
end proc: #end orderrecur (12)

# 2-A-2-2 We give here the procedure for the determination of the recurrence equation satisfied by the complex Fourier coefficients of a given function, without computing
#  the differential equation.
# 
# First part of the procedure for the computation of the Fourier coefficients of trigonometric holonomic functions (in  the general case) without involving the differential equation.
rethd:=proc(f,Ft,P,L,a,b)
local F,t,g,num,num1,numRule,den,den1,denRule,w,ansatz,ANSATZ,vars,l,p,entry,eqs,EQS,sol,ENTRY,i,j,res;
F:=op(0,Ft);
t:=op(1,Ft);
g:=f;
num:=numer(g);
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=denom(g);
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
g:=num/den;
g:=convert(g,trig);
w:=2*Pi/(b-a);
ansatz:=0; ANSATZ:=0;vars:={};
for l from 0 to L do
    for p from 0  to P do
        ansatz:=ansatz+(alpha[p,l]*cos(l*w*t)+beta[p,l]*sin(l*w*t))*diff(g,[t$p]); 
        ANSATZ:=ANSATZ+(alpha[p,l]*cos(l*w*t)+beta[p,l]*sin(l*w*t))*diff(F(t),[t$p]); 
        vars:={op(vars),alpha[p,l],beta[p,l]}; 
        entry:=combine(numer(normal(ansatz)),trig); 
    end do;
end do;
eqs:={coeffs(entry,indets(entry,function))}; 
EQS:=map(term->coeffs(term,t),eqs); 
sol:=solve(EQS,vars); 
ENTRY:=subs(sol,ANSATZ);
for i from 0 to l do 
    for j from 0 to p do  
        ENTRY:=numer(subs(alpha[j,i]=1,beta[j,i]=1,ENTRY));
    od;
od;  
res:=collect(ENTRY,diff,factor);  
if not res=0 then
    print(subs(numRule,denRule,f),"satisfies the trigonometric holonomic differential equation:");
    res:=subs(numRule,denRule,res);
    print(res);
    return subs(numRule,denRule,sol); 
else 
    Error ;
end if; 
end proc: #end rethd; (44)


#  Procedure for the direct computation of a recurrence equation for the complex Fourier 
# coefficients of a given function, without involving the procedure THDE.
RETHd:=proc(f,Ft,cn,a,b) 
local g,F,w,t,pl,L,P,vi,L_Max,P_Max,ansatz,ANSATZ,vars,sol,RE,res,k,l,l1,p1,p;
g:=f;
F:=op(0,Ft);
t:=op(1,Ft);
w:=2*Pi/(b-a);
if type(g,RATTRIG(t)) then
    if  type(RATTRIGtest(g,t)/w,integer) then
        pl:=EfficientPL(subs(t=t/w,g),t);
        L:=op(2,op(1,pl));
        P:=op(2,op(2,pl));
       sol:=rethd(f,F(t),P,L,a,b);
       RE:=add(2*alpha[p,0]*(-1)^p*((-2*n*Pi*I/(b-a))^p*(c(n)-add((-1)^j*(b-a)^j*(I/(2*                           Pi*n))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*w*n*a),j=0..p                          -1))),p=0..P)+add(add((alpha[p,l]+I*beta[p,l])*(2*(n+l)*Pi*I/(b-a))^p*(c(n+l) -add((-1)^j*(b-a)^j*(I/(2*Pi*(n+l)))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*w*(n+l)*a),j=0..p-1))+(alpha[p,l]-I*beta[p,l])*(2*(n-l)*Pi*I/(b-a))^p*(c(n-l)-add((-1)^j*(b-a)^j*(I/(2*Pi*(n-l)))^(j+1)*(subs(t=b,diff(f,[t$j])   )-subs(t=a,diff(f,[t$j])))*exp(-I*w*(n-l)*a),j=0..p-1)),l=1..L),p=0..P) ;
        RE:=subs(sol,RE);
        RE:=simplify(RE) assuming n::integer;
        for p from 0 to P do  
            for l from 0 to L do 
                RE:=numer(subs(alpha[p,l]=1,beta[p,l]=1,RE));  
            od;     
        od; 
        RE:=(collect(RE,c,factor)=0);
            if simplify(subs(t=b,diff(f,[t$0]))  -subs(t=a,diff(f,[t$0])))=0 then 
                print("and its Fourier coefficients satisfy the following homogeneous recurrence equation RE of order",orderrecur(RE,c(n)));
            else  
                print("and its Fourier coefficients satisfy the following inhomogeneous recurrence equation RE of order",orderrecur(RE,c(n))); 
            fi;                         
            return  RE;
    fi;
    vi:=5;
fi;
vi:=5;
L_Max:=vi;
P_Max:=vi;
ansatz:=0;
ANSATZ:=0;
vars:={};
res:=Error;
for k from 0 do
    while res=Error and L_Max<2*vi do
        for l from 0 to L_Max do
            for p from P_Max -vi +1  to P_Max do
                sol:=rethd(g,Ft,p,l,a,b);                       
                if not sol=Error then
                    RE:=add(2*alpha[p1,0]*(-1)^p1*((-2*n*Pi*I/(b-a))^p1*(c(n)-add((-1)^j*(b-a)^j*(I/(2*                            Pi*n))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*w*n*a),j=0..p1 -1))),p1=0..p)+add(add((alpha[p1,l1]+I*beta[p1,l1])*(2*(n+l1)*Pi*I/(b-a))^p1*(c(n+l1)-add((-1)^j*(b-a)^j*(I/(2*Pi*(n+l1)))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*w*(n+l1)*a),j=0..p1-1))+(alpha[p1,l1]-I*beta[p1,l1])*(2*(n-l1)*Pi*I/                    (b-a))^p1*(c(n-l1)-add((-1)^j*(b-a)^j*(I/(2*Pi*(n-l1)))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*w*(n-l1)*a),j=0..p1-1)),l1=1..l),p1=0..p) ;
                    RE:=subs(sol,RE);
                    RE:=simplify(RE) assuming n::integer;
                    for p1 from 0 to p do  
                        for l1 from 0 to l do 
                            RE:=numer(subs(alpha[p1,l1]=1,beta[p1,l1]=1,RE));  
                        od;     
                    od; 
                    RE:=(collect(expand(RE),c,factor)=0);
                    if simplify(subs(t=b,diff(f,[t$0]))  -subs(t=a,diff(f,[t$0])))=0 then 
                        print("and its Fourier coefficients satisfy the following homogeneous recurrence equation RE of order",orderrecur(RE,c(n)));
                    else  
                        print("and its Fourier coefficients satisfy the following inhomogeneous recurrence equation RE of order",orderrecur(RE,c(n))); 
                    fi;                         
                    return  RE;
                else
                    res:=Error;
                fi;     
            od;      
        od; 
        L_Max:=L_Max+vi; 
        P_Max:=P_Max+vi; 
        print(k);
    od;
od;
end proc: #end RETHd (69)

# 2-A-2-3 Conversion of a trigonometric differential equation into a recurrence equation.
# The procedure THDEtoRE converts a trigonometric holonomic differential equation with coefficients in K[cos(t), sin(t)] into a recurrence equation.
THDEtoRE:=proc(DE,f,Ft,cn,a,b)
local DElocal,F,t,g,num,num1,numRule,i,j,den,den1,denRule,c,n,w,P,L,p,l,alpha,beta,Dif,RE,eq,ANSATZ; 
if op(2,DE)= 0 then
DElocal:=op(1,DE);
else DElocal:=DE;
end if;
F:=op(0,Ft);
t:=op(1,Ft);
num:=numer(DElocal);
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=denom(DElocal);
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
DElocal:=num/den;
DElocal:=combine(DElocal);
c:=op(0,cn);
n:=op(1,cn);
w:=2*Pi/(b-a);
P:=PDEtools[difforder](DE,t);
L:=[(op(map((y->y/t,map(x->op(1,x),select(has,indets(DElocal),[cos,sin]))))))];
L:=max(seq(op(i,L)/w,i=1..nops(L)));
L:=simplify(L);
if L<0 then
L:=1;
fi;
if L=-infinity then 
    L:=0;
fi;
for p from P to 0 by -1 do
    for l from 0 to L do 
        DElocal:=subs(diff(F(t),[t$p])=Z^p,DElocal);
        DElocal:=subs(cos(l*w*t)=X^l,DElocal); 
        DElocal:=subs(sin(l*w*t)=Y^l,DElocal);
    od;
od;
for p from 0 to P do 
    for l from 0 to L do
        alpha[p,l]:=coeff(coeff(coeff(DElocal,Z,p),X,l),Y,0);
        beta[p,l]:=coeff(coeff(coeff(DElocal,Z,p),Y,l),X,0);
    od; 
od;
ANSATZ:=0; 
for p from 0 to P do 
    if p=0 then 
        Dif:=F(t); 
    else 
        Dif:=diff(F(t),t$p);
    fi;
    for l from 0 to L do 
        ANSATZ:=ANSATZ+(alpha[p,l]*cos(l*w*t)+beta[p,l]*sin(l*w*t))*Dif;
    od;  
od;
RE:=add(2*alpha[p,0]*c[p](n),p=0..P)+ add(add(c[p](n+l)*(alpha[p,l]+I*beta[p,l])+c[p](n-l)*(alpha[p,l]-I*beta[p,l]),l=1..L),p=0..P) ;
for p from 0 to P do
    for l from -L to L do  
        eq :=c[p](n+l)=(2*(n+l)*Pi*I/(b-a))^p*(c(n+l)-simplify(add((-1)^j*(b-a)^j*(I/(2*(n+l)*Pi))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*(n+l)*2*Pi*b/(b-a)),j=0..p-1)))assuming n::integer; 
        RE := subs(eq, RE) ;
    od;
od;
RE:=collect(numer(normal(RE)),c,factor) ;
RE:=subs(numRule,denRule,RE);
RE:=collect(expand(RE),c,factor);
if simplify(subs(t=b,diff(f,[t$0]))  -subs(t=a,diff(f,[t$0])))=0 and simplify(subs(t=b,diff(f,[t$1]))-subs(t=a,diff(f,[t$1])))=0 then 
    print("and its Fourier coefficients satisfy the following homogeneous recurrence equation of order", orderrecur(RE,c(n)));
else  
    print("and its Fourier coefficients satisfy the following inhomogeneous recurrence equation of order", orderrecur(RE,c(n))); 
fi; 
RE=0;
end: #THDEtoRE;(72)

# 2-A-2-4 Procedure for the computation of a recurrence equation for the complex Fourier coefficients 
# of a given function $f$, via the procedure THDE, i.e. we first compute the differential equation, then convert it into a recurrence equation.
RETH:=proc(f,Ft,cn,a,b)
local localde,w,c;
w:=2*Pi/(b-a);
localde:=THDE(f,Ft,w);
print(localde);
THDEtoRE(localde,f,F(t),cn,a,b);
end proc:#end RETH(7)

# 2-A-2-5 Procedure for the computation of a recurrence equation for rational trigonometric functions
#  with constant coefficients. Note that the input function must be a rational trigonometric function.
RERatTH:=proc(f,t,cn)
local h,ti,c,n,g,num,den,N,D,coef1,coef2,RE,d;
h:=convert(f,trig);
ti:=t;
if not type(h,RATTRIG(ti)) then
    return ("The input function",f," is not (or cannot be brought into) a rational trigonometric function");
fi;
c:=op(0,cn);
n:=op(1,cn);
g:=subs(cos(ti)=(z+1/z)/2,h);
g:=normal(subs(sin(ti)=(z-1/z)/(2*I),g));
num:=numer(normal(g));
den:=denom(normal(g));
N:=degree(num);
D:=degree(den);
coef1:=[seq(coeff(num,z,i),i=0..N)];
coef2:=[seq(coeff(den,z,i),i=0..D)];
RE:=add(coef2[i+1]*c(n-i),i=0..D);
d:=orderrecur(RE,c(n));
if d=0 then
    return("the denomination of the function",f,"muss be a trigonometric polynomial of variable",t," of at least of first degree");
fi;
print("The Fourier coefficients of",f,"satisfy the constant coefficients recurrence equation of order",   orderrecur(RE,c(n)))=0; 
RE;
end proc: #end RERatTH (25)

# 2-A-3  Solving of the recurrence equations and deduction of the Fourier coefficients.

# 2-A-3-1The following procedure solves recurrence equations, searching preferably closed form solutions.
 
SolRE:=proc(RE,f,cn,a,b)
local c,n,p,A,B,sol;
c:=op(0,cn);
n:=op(1,cn);
p:=orderrecur(RE,c(n));
if p=0 then
    print("The complex Fourier coefficients of ",f," on the interval",[a,b],"are:");
    return c(n)=solve(RE,c(n));
fi;
A:=[seq(simplify(1/(b-a)*int(f*exp(-I*s*2*Pi/(b-a)*t),t=a..b)),s=0..p-1)];
if has(A,t) then 
    return print("The solve of the recurrence equation",RE,c(0)=op(1,A),"cannot be completed, since enough initial values cannot be computed.");
fi;
B:={seq(c(i)=op(i+1,A),i=0..p-1)}; 
sol:=LREtools[hypergeomsols](RE, c(n), B);
if has({sol},RootOf) then
    return print("There is no solution in closed form of the recurrence equation",RE,"with initial  values",seq(c(i)=op(i+1,A),i=0..p-1)); 
fi;  
if not {sol}={} then 
    if p<2 then
        print("Solving it with the initial value",seq(c(i)=op(i+1,A),i=0..p-1)," we get:");
    else
        print("Solving it with the initial values",seq(c(i)=op(i+1,A),i=0..p-1)," we get:");
    fi;
    return c(n)=sol;
fi; 
sol:=rsolve({RE,op(B)}, {c});
end proc: #end SolRE;(28)

# 2-A-3-2 Procedure for the computation of the Fourier coefficients of trigonometric holonomic functions (in the general case)  without involving the procedure THDEtoRE.

FouCoeffTHd:=proc(f,Ft,cn,a,b)
local RE;
RE:=RETHd(f,Ft,cn,a,b); 
print(RE);
SolRE(RE,f,c(n),a,b);
end proc:#end  FouCoeffTHd (6)

# 2-A-3-3 Procedure for the computation of the Fourier coefficients of trigonometric holonomic functions (in  the general case).
FouCoeffTH:=proc(f,Ft,cn,a,b)
local w,localde, localre;
w:=2*Pi/(b-a);
localde:=THDE(f,Ft,w);
print(localde);
localre:=THDEtoRE(localde,f,Ft,cn,a,b);
print(localre);
SolRE(localre,f,cn,a,b);
end proc:#end FouCoeffTH(9)

# Section B:  This part is devoted to the computations involving a differential equation with coefficients in K[e^{-iwt}, e^{iwt}]]
# Section B is divided into three sections, namely:  B-1 Determination of trigonometric differential equations.  B-2 Computation of a recurrence equation for the Fourier coefficients B-3 Solving of the recurrence equations and deduction of the Fourier coefficients. 
# 2-B-1 Computation of trigonometric differential equations. 
# 2-B-1-1 The following procedure determines a trigonometric holonomic differential equation 
# with coefficients in K[e^{it}] of exact degree P and with exact L.
expthdePL1:=proc(f,Ft,P,L,w)
local F,t,g,num,num1,numRule,den,den1,denRule,ansatz,ANSATZ,vars,p,l,dif,DIF,gamma,delta,eqs,EQS,sol,res; 
F:=op(0,Ft);
t:=op(1,Ft);
g:=f;
if not has(g,t) then
    return exp(I*w*t)*diff(F(t),t)=0;
fi;
num:=numer(g);
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=denom(g);
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
g:=num/den;
g:=convert(g,exp);
ansatz:=0;
ANSATZ:=0;
vars:={};
for p from 0 to P do
    if p=0 then
        dif:=g;
        DIF:=F(t);                         
    else
        dif:=diff(g,t$p);
        DIF:=diff(F(t),t$p)   
    fi; 
    for l from 0 to L do 
        ansatz:=ansatz+(gamma[p,l]*exp(-I*l*w*t)+delta[p,l]*exp(I*l*w*t))*dif;  
        ANSATZ:=ANSATZ+(gamma[p,l]*exp(-I*l*w*t)+delta[p,l]*exp(I*l*w*t))*DIF; 
        vars:={op(vars),gamma[p,l],delta[p,l]}; 
    od; 
od;   
ansatz:=(numer(normal(ansatz))); 
ansatz:=expand(ansatz);
eqs:={coeffs(ansatz,indets(ansatz,function))};
EQS:=map(term->coeffs(term,t),eqs); 
sol:=solve(EQS,vars);     
ANSATZ:=subs(sol,ANSATZ);
for l from 0 to L do  
    for p from 0 to P do 
        ANSATZ:=numer(subs(gamma[p,l]=1,delta[p,l]=1,ANSATZ));  
    od;     
od;   
res:=collect(ANSATZ,diff,factor);  
if not res=0 then
    print(subs(numRule,denRule,f),"satisfies the trigonometric holonomic differential equation:");
    res:=subs(numRule,denRule,res);
    return res; 
else Error  ;
fi; 
end: #end expthdePL1 (54)

# The following procedure determines a trigonometric holonomic differential equation of trigonometric holonomic functions
# with coefficients in K[e^{-it},e^{it}] of exact degree P and with exact L.
expthdePL2:=proc(f,Ft,P,L,w)
local  F,t,g,num,num1,numRule,den,den1,denRule,ansatz,ANSATZ,vars,p,l,dif,DIF,delta,eqs,EQS,sol,res; 
F:=op(0,Ft);
t:=op(1,Ft);
g:=f;
if not has(g,t) then
    return exp(I*w*t)*diff(F(t),t)=0;
fi;
num:=numer(g);
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=denom(g);
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
g:=num/den;
g:=convert(g,exp);
ansatz:=0;
ANSATZ:=0;
vars:={};
for p from 0 to P do
    if p=0 then 
        dif:=g;
        DIF:=F(t);                         
    else 
        dif:=diff(g,t$p);
        DIF:=diff(F(t),t$p)   
    fi; 
    for l from 0 to L do 
        ansatz:=ansatz+(delta[p,l]*exp(I*l*w*t))*dif;  
        ANSATZ:=ANSATZ+(delta[p,l]*exp(I*l*w*t))*DIF; 
        vars:={op(vars),delta[p,l]}; 
    od; 
od;   
ansatz:=(numer(normal(ansatz))); 
ansatz:=expand(ansatz);
eqs:={coeffs(ansatz,indets(ansatz,function))};
EQS:=map(term->coeffs(term,t),eqs); 
sol:=solve(EQS,vars);   
ANSATZ:=subs(sol,ANSATZ);
for l from 0 to L do  
    for p from 0 to P do 
        ANSATZ:=numer(subs(delta[p,l]=1,ANSATZ));  
    od;     
od; 
res:=collect(ANSATZ,diff,factor);  
if not res=0 then
    print(subs(numRule,denRule,f),"satisfies the trigonometric holonomic differential equation:");
    res:=subs(numRule,denRule,res);
    return res; 

else 
    Error  ;
fi; 
end: #end expthdePL2 (53)

# The following procedure combines the 2 previous and determines a trigonometric holonomic differential equation, satisfied by trigonometric holonomic functions,
# with coefficients in:  K[e^{iwt}] if L is odd, or in  K[e^{-iwt},e^{iwt}] otherwise, of exact degree P and with exact L.
expthdePL:=proc(f,Ft,P,L,w)
local F,t,localde1,localde2,res ;
F:=op(0,Ft);
t:=op(1,Ft);
if not has(f,t) then
    return exp(I*w*t)*diff(F(t),t)=0;
fi;
localde1:=expthdePL2(f,F(t),P,2*L-1,w);
if not localde1=Error then 
    res:=localde1;
    return res; 
fi;
localde2:=expthdePL1(f,F(t),P,L,w);
if not localde2=Error then 
    res:= localde2;
    return res;
fi;
res:= Error ;
res;
end proc: #end expthdePL(20)

# 2-B-1-2 The following procedure determines a differential equation with coefficients in K[e^{iwt}] or in K[e^{-iwt}, e^{iwt}].  It first checks if the input can be brought into a rational trigonometric function and computes the Efficient P and L if it is the case. 
ExpTHDE:=proc(f,Ft,w)
local F,t,g,res,v1,L1,P1,k,l,p,localde1,localde2,pl,P,L,vi ;
F:=op(0,Ft);
t:=op(1,Ft);
g:=convert(f,trig);
if not has(g,t) then
    return exp(I*w*t)*diff(F(t),t)=0;
fi;
if type(g,RATTRIG(t)) then 
    if  type(RATTRIGtest(g,t)/w,integer) then
        pl:=EfficientPL(subs(t=t/w,g),t);
        L:=op(2,op(1,pl));
        #P:=op(2,op(2,pl));
         P:=1;
        res:=Error ;
        vi:=3;        
        if type(L,even) then 
            L:=L/2;
        else
            L:=(L+1)/2;
        fi;
        P1:=P+vi;
        L1:=L+vi;
        for k from 1 do
            while  res=Error do
                for l from L to L1 do
                    for p from P-1 to P1 do
                        if type(l,odd) then
                            localde2:=expthdePL2(f,F(t),p,l,w);
                            if not localde2=Error then 
                                res:= localde2;
                                return res; 
                            fi;
                        else
                            localde1:=expthdePL1(f,F(t),p,l/2,w);
                            if not localde1=Error then 
                                res:= localde1;
                                return res;
                            fi;
                        fi;
                        res:= Error;
                        res;
                    od; 
                od; 
                L1:=L1+v1;
                P1:=P1+v1;
            od;
        od;
    fi;
vi:=5;
fi;
res:=Error ;
v1:=5; 
L1:=v1;
P1:=v1;
for k from 1 do
    while  res=Error do
        for l from 0 to L1 do
            for p from P1-v1+1 to P1 do
                if type(l,odd) then
                            localde2:=expthdePL2(f,F(t),p,l,w);
                            if not localde2=Error then 
                                res:= localde2;
                                return res; 
                            fi;
                        else
                            localde1:=expthdePL1(f,F(t),p,l/2,w);
                            if not localde1=Error then 
                                res:= localde1;
                                return res;
                            fi;
                        fi;
                res:= Error;
                res;
            od; 
        od; 
        L1:=L1+v1;
        P1:=P1+v1;
    od;
od;
end proc : #end ExpTHDE(81)


# 2-B-2 Determination of recurrence equations.

# 2-B-2-1 Direct computation of the recurrence equation of the complex Fourier coefficients via a differential equation with coefficients 
#  in K[exp{-iwt},exp(iwt}],  without computing explicitly a differential equation.
# 
# part 1
expthre1d:=proc(f,Ft,P,L,w)
local F,t,g,num,num1,numRule,den,den1,denRule,ansatz,ANSATZ,vars,p,l,dif,DIF,eqs,EQS,sol,res; 
F:=op(0,Ft);
t:=op(1,Ft);
g:=f;
#g:=convert(f,exp);
num:=numer(g);
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=denom(g);
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
g:=num/den;
g:=convert(g,exp);
ansatz:=0;
ANSATZ:=0;
vars:={};
for p from 0 to P do
    for l from 0 to L do 
        ansatz:=ansatz+(gamma[p,l]*exp(-I*l*w*t)+delta[p,l]*exp(I*l*w*t))*diff(g,[t$p]);  
        ANSATZ:=ANSATZ+(gamma[p,l]*exp(-I*l*w*t)+delta[p,l]*exp(I*l*w*t))*diff(F(t),[t$p])   ; 
        vars:={op(vars),gamma[p,l],delta[p,l]}; 
    od; 
od;   
ansatz:=(numer(normal(ansatz))); 
ansatz:=expand(ansatz);
eqs:={coeffs(ansatz,indets(ansatz,function))};
EQS:=map(term->coeffs(term,t),eqs); 
sol:=solve(EQS,vars);     
ANSATZ:=subs(sol,ANSATZ);
for l from 0 to L do  
    for p from 0 to P do 
        ANSATZ:=numer(subs(gamma[p,l]=1,delta[p,l]=1,ANSATZ));  
    od;     
od;   
res:=collect(ANSATZ,diff,factor);  
if not res=0 then
    print(subs(numRule,denRule,f),"satisfies the following differential equation:");
    res:=subs(numRule,denRule,res);
    print(res);
    return subs(numRule,denRule,sol);
else Error  ;
fi; 
end: #end expthdePL1; (46)

#  part 2

expthre2d:=proc(f,Ft,P,L,w)
local F,t,g,num,num1,numRule,den,den1,denRule,ansatz,ANSATZ,vars,p,l,dif,DIF,eqs,EQS,sol,res; 
F:=op(0,Ft);
t:=op(1,Ft);
g:=f;
#g:=convert(f,exp);
num:=numer(g);
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=denom(g);
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
g:=num/den;
g:=convert(g,exp);
ansatz:=0;
ANSATZ:=0;
vars:={};
for p from 0 to P do
    for l from 0 to L do 
        ansatz:=ansatz+(delta[p,l]*exp(I*l*w*t))*diff(g,[t$p]); 
        ANSATZ:=ANSATZ+(delta[p,l]*exp(I*l*w*t))*diff(F(t),[t$p]);  
        vars:={op(vars),delta[p,l]}; 
    od; 
od;   
ansatz:=(numer(normal(ansatz))); 
ansatz:=expand(ansatz);
eqs:={coeffs(ansatz,indets(ansatz,function))};
EQS:=map(term->coeffs(term,t),eqs); 
sol:=solve(EQS,vars);   
ANSATZ:=subs(sol,ANSATZ);
for l from 0 to L do  
    for p from 0 to P do 
        ANSATZ:=numer(subs(delta[p,l]=1,ANSATZ));  
    od;     
od; 
res:=collect(ANSATZ,diff,factor);  
if not res=0 then
    print(subs(numRule,denRule,f),"satisfies the following differential equation:");
    res:=subs(numRule,denRule,res);
    print(res);
    return subs(numRule,denRule,sol); 
else 
    Error;
fi; 
end: #end expthdePL2;(47)

# part 3, which puts together the first  two parts.
expthred:=proc(f,Ft,w)
local F,t,g,res,v1,L1,P1,k,l,p,localde1,localde2,pl,P,L,vi ;
F:=op(0,Ft);
t:=op(1,Ft);
g:=convert(f,trig);
if type(g,RATTRIG(t)) then 
    if  type(RATTRIGtest(g,t)/w,integer) then
        pl:=EfficientPL(subs(t=t/w,g),t);
        L:=op(2,op(1,pl));
        P:=op(2,op(2,pl));
        res:=Error;
        vi:=3;  
        if type(L,even) then 
            L:=L/2;
        else
            L:=(L+1)/2;
        fi;
        P1:=P+vi;
        L1:=L+vi;
        for k from 1 do
            while  res=Error do
                for l from L to L1 do
                    for p from P to P1 do
                        if type(l,odd) then
                            localde2:=expthre2d(f,F(t),p,l,w);
                            if not localde2=Error then 
                                res:= localde2;
                                return (res,p,l); 
                            fi;
                        else
                            localde1:=expthre1d(f,F(t),p,l/2,w);
                            if not localde1=Error then 
                                res:= localde1;
                                return (res,p,l/2);
                            fi;
                        fi;
                        res:= Error;
                        res;
                    od; 
                od; 
                L1:=L1+v1;
                P1:=P1+v1;
            od;
        od;
    fi;
    vi:=5;
fi;
res:=Error ;
v1:=5; 
L1:=v1;
P1:=v1;
for k from 1 do
    while  res=Error do
        for l from 0 to L1 do
            for p from P1-v1+1 to P1 do
                localde1:=expthre2d(f,F(t),p,2*l-1,w);
                if not localde1=Error then 
                    res:= localde1;
                    return (res,p,2*l-1); 
                fi;
                localde2:=expthre1d(f,F(t),p,l,w);
                if not localde2=Error then 
                    res:= localde2;
                    return (res,p,l);
                fi;
                res:= Error;
                res;
            od; 
        od; 
        L1:=L1+v1;
        P1:=P1+v1;
    od;
od;
end proc : #end expthred;(74)

# Procedure for the direct computation of a recurrence equation satisfied by the complex Fourier coefficients of a trigonometric holonomic function, solution of a differential equation with coefficients in K[exp{-iwt},exp(iwt}], without involving the DE.
ExpTHREd:=proc(f,Ft,cn,a,b) 
local g,F,w,T,t,vi,L_Max,P_Max,ansatz,ANSATZ,vars,sol,RE,res,k,l,l1,p1,p,P,L,rule1,rule2,rule3;
rule1:=gamma(i::anything,j::anything)=0;
rule2:=gamma(i::anything,j::anything)=1;
rule3:=delta(i::anything,j::anything)=1;
g:=convert(f,trig );
F:=op(0,Ft);
t:=op(1,Ft);
w:=2*Pi/(b-a);
T:=2*Pi/w;
sol:=[expthred(f,Ft,w)];
P:=op(2,sol);
L:=op(3,sol);
sol:=op(op(1,sol));  
if not sol= Error then 
    RE:=add(add(delta[p,l]*(2*(n-l)*Pi*I/T)^p*(c(n-l)-add((-1)^j*(T)^j*(I/(2*(n-l)*Pi))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*(n-l)*w*a),j=0..p-1))+gamma[p,l]*(2*(n+l)*Pi*I/T)^p*(c(n+l)-add((-1)^j*(T)^j*(I/(2*(n+l)*Pi))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*(n+l)*w*a),j=0..p-1)),l=0..L),p=0..P);
    RE:=subs(sol,RE);
    RE:=simplify(RE) assuming n::integer;
    if has({sol}, gamma) then 
        RE:=applyrule([rule2, rule3],RE);                        
        RE:=collect(expand(RE),c,factor)=0;
    else
        RE:=applyrule([rule1, rule3],RE)=0;   
    fi;
    if simplify(subs(t=b,diff(f,[t$0])) -subs(t=a,diff(f,[t$0])))=0 then 
        print("and its Fourier coefficients satisfy the following homogeneous recurrence equation RE of order",orderrecur(RE,c(n)));
    else  
        print("and its Fourier coefficients satisfy the following inhomogeneous recurrence equation RE of order",orderrecur(RE,c(n))); 
    fi; 
    return RE;
fi; 
res:= Error;
end proc: #end ExpTHREd (33)


# 2-B-2-2 The procedure ExpTHDEtoRE converts a trigonometric holonomic differential equation with coefficients in K[e^{-it},e^{it}] into a recurrence equation.
ExpTHDEtoRE:=proc(DE,f,Ft,cn,a,b)
local DElocal,F,t,g,num,num1,numRule,den,den1,denRule,c,n,w,T,P,L1,L2,L,p,l,gamma,delta,RE,dif,ANSATZ,DIF,maxi,mini;
if op(2,DE)= 0 then
DElocal:=op(1,DE);
else DElocal:=DE;
end if;
F:=op(0,Ft);
t:=op(1,Ft);
num:=simplify(numer(DElocal));
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=simplify(denom(DElocal));
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
DElocal:=num/den;
c:=op(0,cn);
n:=op(1,cn);
w:=2*Pi/(b-a);
T:=2*Pi/w;
P:=PDEtools[difforder](DElocal,t);
L1:=map((y->y/t*I,map(x->op(1,x),select(has,indets(DElocal),exp))));
L2:=map((y->y/t*1/I,map(x->op(1,x),select(has,indets(DElocal),exp))));
L:=L1 union L2;
L:=max(seq(op(i,L)/w,i=1..nops(L)));
if L=-infinity then 
    L:=0;
fi;
DElocal:=simplify(DElocal);
for p from P to 0 by -1 do
    for l from 0 to L do 
        DElocal:=subs(diff(F(t),[t$p])=Z^(p+1),DElocal);
        DElocal:=subs(exp(-I*l*w*t)=X^l,DElocal); 
        DElocal:=subs(exp(I*l*w*t)=Y^l,DElocal);
    od;
od;
for p from 1 to P+1 do 
    for l from 0 to L do
        delta[p-1,l]:=coeff(coeff(coeff(DElocal,Z,p),Y,l),X,0);
        DElocal:=DElocal-delta[p-1,l]*Z^p*Y^l;
        gamma[p-1,l]:=coeff(coeff(coeff(DElocal,Z,p),X,l),Y,0);
        DElocal:=DElocal-gamma[p-1,l]*Z^p*X^l;
    od; 
od;
ANSATZ:=0;
for p from 0 to P do
    for l from 0 to L do  
        ANSATZ:=ANSATZ+(gamma[p,l]*exp(-I*l*w*t)+delta[p,l]*exp(I*l*w*t))*diff(F(t),[t$p]);
    od; 
od;  
RE:=add(add(delta[p,l]*(2*(n-l)*Pi*I/T)^p*(c(n-l)-add((-1)^j*(T)^j*(I/(2*(n-l)*Pi))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*(n-l)*w*a),j=0..p-1))+gamma[p,l]*(2*(n+l)*Pi*I/T)^p*(c(n+l)-add((-1)^j*(T)^j*(I/(2*(n+l)*Pi))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*(n+l)*w*a),j=0..p-1)),l=0..L),p=0..P);
RE:=(collect(numer(normal(simplify(RE))),c,factor)) ;
RE:=subs(numRule,denRule,RE);
RE:=collect(expand(RE),c,factor);
if simplify(subs(t=b,diff(f,[t$0]))-subs(t=a,diff(f,[t$0])))=0 then 
    print("and its Fourier coefficients satisfy the following homogeneous recurrence equation of order", orderrecur(RE,c(n)));
else  
    print("and its Fourier coefficients satisfy the following inhomogeneous equation of order", orderrecur(RE,c(n))); 
fi; 
RE=0; 
end: #end ExpTHDEtoRE;(62)
# The following procedure computes a recurrence equation satisfied by the complex Fourier coefficients of a trigonometric holonomic function, via the search of a linear differential equation with coefficients in K[e^{-iwt}, e^{-iwt}].
ExpRETH:=proc(f,Ft,cn,a,b)
local F,t,c,n,w,localde;
F:=op(0,Ft);
t:=op(1,Ft);
c:=op(0,cn);
n:=op(1,cn);
w:=2*Pi/(b-a);
localde:=ExpTHDE(f,F(t),w);
print(localde);
ExpTHDEtoRE(localde,f,F(t),c(n),a,b);
end proc: #end ExpRETH(11)


# 2-B-2-3 Procedure for the computation of Fourier coefficients of trigonometric holonomic differential equation with coefficients in K[e^{-iwt}, e^{-iwt}].
# -  involving the procedure  ExpTHDE and ExpTHDEtoRE 

FFSexp:=proc(f,Ft,cn,a,b)
local w,localde, localre;
w:=2*Pi/(b-a);
localde:=ExpTHDE(f,Ft,w);
print(localde);
localre:=ExpTHDEtoRE(localde,f,Ft,cn,a,b);
print(localre);
SolRE(localre,f,cn,a,b);
end proc:#end FFSexp;(9)

# 
# - without  involving the procedure  ExpTHDE and ExpTHDEtoRE 

FFSexpd:=proc(f,Ft,cn,a,b)
local localre;
localre:=ExpTHREd(f,Ft,cn,a,b);
print(localre);
SolRE(localre,f,cn,a,b);
end proc: #end FFSexpd(6)


#  Part 3  Procedures not depending on the form of the differential equation.

# Section A)  Procedures which reverses THDEtoRE and ExpTHDEtoRE, i.e which searched if for a given holonomic recurrence equation there exist a corresponding trigonometric differential equation.

# The following procedure determines the maximum of the degrees of the coefficients of a holonomic recurrence equation.
deg:=proc(RE,cn)
local localre,c,n,eqs,l,L,X,j;
c:=op(0,cn);
n:=op(1,cn);
if op(2,RE)=0 then 
    localre:=op(1,RE);
else
    localre:=RE;
fi;
L:=orderrecur(localre,c(n));
localre:=collect(localre,c,factor);
    for l from 0 to L  do
        localre:=subs(c(n+l)=X^(n+l),localre);
        localre:=subs(c(n-l)=X^(n-l),localre);
    od;
 eqs:={seq(collect(coeff(localre,X^(n+l)),n,factor),l=-L..L)};
eqs:=decprod(decsum(eqs));
eqs:=select(has,eqs,n);
max({seq(degree(op(i,eqs)),i=1..nops(eqs))});
end proc: # end deg (20)
# A-1  Case of THDEtoRE
# 
# First part of the algorithm for the conversion of a holonomic recurrence equation  into a trigonometric holonomic differential equation. 
reversethdetore:=proc(RE,cn,p1,a,b)
local T,w,localre,c,n,P,vars,equa,L,ANSATZ,re,p,l,coef,eqs,nop,i,sol;
with(PDEtools):
T:=b-a;
w:=2*Pi/T;
if op(2,RE)=0 then 
    localre:=op(1,RE);
else
    localre:=RE;
fi;
c:=op(0,cn);
n:=op(1,cn);
P:=p1;
vars:={};
equa:={};
L:=orderrecur(localre,cn);

if type(L,even) then
    L:=L/2;
    ANSATZ:=0;
    re:=sum(2*alpha[p,0]*(-1)^p*(2*n*Pi/(I*T))^p*c(n),p=0..P)+
    sum(sum((alpha[p,l]+I*beta[p,l])*(2*(n+l)*Pi*I/T)^p*c(n+l)+
    (alpha[p,l]-I*beta[p,l])*(2*(n-l)*Pi*I/T)^p*c(n-l),l=1..L),p=0..P);
    re:=shiff(re,c(n))-shiff(localre,c(n));
    re:=collect(re,c,factor);
    for l from 0 to 2*L  do
        re:=subs(c(n+l)=X^(n+l),re);
    od;
    eqs:={seq(collect(coeff(re,X^(n+l)),n,factor),l=0..2*L)};
    nop:=nops(eqs);
    for i from 1 to nop do
        equa:={op(equa),coeffs(eqs[i],n)};
    od;
    for p from 0 to P do
        for l from 0 to L do
            vars:={op(vars),alpha[p,l],beta[p,l]};
            ANSATZ:=ANSATZ+(alpha[p,l]*cos(l*w*t)+beta[p,l]*sin(l*w*t))*diff(F(t),[t$p]); 
        od;
    od;
else 
L:=(L+1)/2;
    ANSATZ:=0;
    re:=sum(2*alpha[p,0]*(-1)^p*(2*n*Pi/(I*T))^p*c(n),p=0..P)+
    sum(sum((alpha[p,l]+I*beta[p,l])*(2*(n+l)*Pi*I/T)^p*c(n+l)+
    (alpha[p,l]-I*beta[p,l])*(2*(n-l)*Pi*I/T)^p*c(n-l),l=1..L),p=0..P);
   re:=shiff(re,c(n))-shiff(localre,c(n));
    re:=collect(re,c,factor);
    for l from 0 to 2*L  do
        re:=subs(c(n+l)=X^(n+l),re);
    od;
    eqs:={seq(collect(coeff(re,X^(n+l)),n,factor),l=0..2*L)};
    nop:=nops(eqs);
    for i from 1 to nop do
        equa:={op(equa),coeffs(eqs[i],n)};
    od;
    for p from 0 to P do
        for l from 0 to L do
            vars:={op(vars),alpha[p,l],beta[p,l]};
            ANSATZ:=ANSATZ+(alpha[p,l]*cos(l*w*t)+beta[p,l]*sin(l*w*t))*diff(F(t),[t$p]); 
        od;
    od;
fi;
sol:=solve(equa, vars);
if not {sol}={} then 
    ANSATZ:=subs(sol,ANSATZ);
    return collect(numer(normal(ANSATZ)),diff,factor);
fi;
no
end proc: #end reversethdetore (69)

# Algorithm for the conversion of a holonomic recurrence equation into a trigonometric holonomic differential equation with coefficients in K[cos(wt),sin(wt)].

ReverseTHDEtoRE:=proc(RE,cn,a,b)
local p1,p,localre;
p1:=deg(RE,cn); 
for p from p1 to p1+5 do
    localre:=reversethdetore(RE,cn,p,a,b);
        if not localre=no then 
            return localre;
        fi;
od;
 print("There is no trigonometric differential equation corresponding to",RE);
end proc: #end ReverseTHDEtoRE (11)

# A-2  Case of ExpTHDEtoRE
# First part of the algorithm for the conversion of a holonomic recurrence equation
#  into a trigonometric holonomic differential equation with coefficients in K[exp(-Iwt), exp(Iwt)].
reverseexpthdetore:=proc(RE,cn,p1,a,b)
local T,w,localre,c,n,P,vars,equa,L,ANSATZ,re,p,l,coef,eqs,nop,i,sol,ansatz,dg;
with(PDEtools):
T:=b-a;
w:=2*Pi/T;
if op(2,RE)=0 then 
    localre:=op(1,RE);
else
    localre:=RE;
fi;
c:=op(0,cn);
n:=op(1,cn);
P:=p1;
vars:={};
equa:={};
L:=orderrecur(localre,cn);
re:=add(add(delta[p,l]*(2*(n-l)*Pi*I/T)^p*(c(n-l)-add((-1)^j*(T)^j*(I/(2*(n-l)*Pi))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*(n-l)*w*a),j=0..p-1))+gamma[p,l]*(2*(n+l)*Pi*I/T)^p*(c(n+l)-add((-1)^j*(T)^j*(I/(2*(n+l)*Pi))^(j+1)*(subs(t=b,diff(f,[t$j]))-subs(t=a,diff(f,[t$j])))*exp(-I*(n+l)*w*a),j=0..p-1)),l=0..L),p=0..P);
ANSATZ:=0;
re:=re-localre;
re:=collect(re,c,factor);
    for l from 0 to L  do
        re:=subs(c(n+l)=X^(n+l),re);
        re:=subs(c(n-l)=X^(n-l),re);
    od;
    eqs:={seq(collect(coeff(re,X^(n+l)),n,factor),l=-L..L)};
    nop:=nops(eqs);
    for i from 1 to nop do
        equa:={op(equa),coeffs(eqs[i],n)};
    od;
    for p from 0 to P do
        for l from 0 to L do 
            #ansatz:=ansatz+(gamma[p,l]*exp(-I*l*w*t)+delta[p,l]*exp(I*l*w*t))*diff(f,[t$p]);  
            ANSATZ:=ANSATZ+(gamma[p,l]*exp(-I*l*w*t)+delta[p,l]*exp(I*l*w*t))*diff(F(t),[t$p]); 
            vars:={op(vars),gamma[p,l],delta[p,l]}; 
        od;
    od;   
    sol:=solve(equa, vars);
    if not {sol}={} then 
        ANSATZ:=subs(sol,ANSATZ);
    return collect(numer(normal(ANSATZ)),diff,factor);
    fi;
    no;
end proc: #end reverseexpthdetore (43)

# Algorithm for the conversion of a holonomic recurrence equation
#  into a trigonometric holonomic differential equation with coefficients in K[exp(-Iwt), exp(Iwt)].
# 
ReverseExpTHDEtoRE:=proc(RE,cn,a,b)
local p1,p,localre;
p1:=deg(RE,cn); 
for p from p1 to p1+5 do
    localre:=reverseexpthdetore(RE,cn,p,a,b);
        if not localre=no then 
            return localre;
        fi;
od;
print("There is no trigonometric differential equation corresponding to",RE);
end proc: # end ReverseExpTHDEtoRE(11)

# 
# Section B  Procedure for the computation of the trigonometric holonomic differential equation  satisfied by the 
# sum (product ) of the solutions of two trigonometric holonomic differential equations.
#  B-1- Detection of the variable in a given differential equation
DEvariable:=proc(DE)
local localde,set0,set1,i,aa,j,Ft,set2,set3;
if op(2,DE)= 0 then
localde:=op(1,DE);
else localde:=DE;
end if;
set0:=indets(localde) ;
set0:=select(has,set0, diff);
if set0={} then 
return (DeOfOrderZero);
fi;
set1:={};
for i from 1 to nops(set0) do
        aa:=op(i,set0);
            while has(op(1,aa),diff)  do
                aa:=op(1,aa);
            od;
        set1:={op(set1), op(1,aa)};
od;
for i from 1 to nops(set1) do
    for j from i+1 to nops(set1) do
        if not op(i,set1)=op(j,set1) then
        return Error;
        fi;
    od;
od;
Ft:=(op(1,set1));
set2:={op(localde)};
set3:={};
for i from 1 to nops(set2) do
    if type(op(i,set2),whattype(sqrt(2))) or nops(op(i,set2))=1 then 
        set3:={op(set3),op(i,set2)};
   else
        set3:={op(set3),select(has,(op(i,set2)),Ft)};
    fi;
od;
set3:=select(has,set3,Ft);
for i from 1 to nops(set3) do
    if type(op(i,set3),whattype(sqrt(2))) or nops(op(i,set3))>1 then 
        return Error;
    fi;
    return Ft;
od;
end proc: #endDEvariable (44)

# This procedure does the same thing as the previous, but return a justification in the case where the output should be false.
Diffvariable:=proc(DE)
local localde,set0,set1,i,aa,j,Ft,set2,set3;
if op(2,DE)= 0 then
localde:=op(1,DE);
else localde:=DE;
end if;
set0:=indets(localde) ;
set0:=select(has,set0, diff);
if set0={} then 
return (DeOfOrderZero);
fi;
set1:={};
for i from 1 to nops(set0) do
        aa:=op(i,set0);
            while has(op(1,aa),diff)  do
                aa:=op(1,aa);
            od;
        set1:={op(set1), op(1,aa)};
od;
for i from 1 to nops(set1) do
    for j from i+1 to nops(set1) do
        if not op(i,set1)=op(j,set1) then
        return (DE,"is a differential equation on more than one variable: \n",op(i,set1),"and",op(j,set1),"for example");
        fi;
    od;
od;
Ft:=(op(1,set1));
set2:={op(localde)};
set3:={};
for i from 1 to nops(set2) do
    if type(op(i,set2),whattype(sqrt(2))) or nops(op(i,set2))=1 then 
        set3:={op(set3),op(i,set2)};
   else
        set3:={op(set3),select(has,(op(i,set2)),Ft)};
    fi;
od;
set3:=select(has,set3,Ft);
for i from 1 to nops(set3) do
    if type(op(i,set3),whattype(sqrt(2))) or nops(op(i,set3))>1 then 
        return (DE,"is not linear");
    fi;
    return Ft;
od;
end proc: #endDiffvariable (45)

# The following procedure determines a  satisfied by the sum of the solutions of 2  differential equations DE1 and DE2.
# Note that this procedure does not care whether the input differential equations are trigonometric holonomic or not.
# But the corresponding functions muss be of the same arguments. ( t or x for example)

SumDE:=proc(DE1,DE2,Ft)
local f,t,localDE1,localDE2,M,N,k,vars1,vars2,vars,var,A,r,s,ANSATZ,eqs1,eqs2,ansatz,
v,l,p,coeff1,coeff2,coeffs,sol,rule,F,G,u,i,j;
f:=op(0,Ft);
t:=op(1,Ft);
localDE1:=DE1;
localDE2:=DE2;
M:=PDEtools[difforder](localDE1,t);
N:=PDEtools[difforder](localDE2,t);
DEvariable(localDE1);
DEvariable(localDE2);
for k from max(M,N) to M+N do
    vars1:={seq(diff(F(t),[t$i]),i=0..k)};
    vars2:={seq(diff(G(t),[t$j]),j=0..k)};
    vars:=vars1 union vars2;
    var:={seq(A[r],r=0..k)};
    localDE1:=subs(op(0,DEvariable(localDE1))=F,localDE1);
    localDE2:=subs(op(0,DEvariable(localDE2))=G,localDE2);
    ANSATZ:=sum(A[s]*diff(f(t),[t$s]),s=0..k);
    eqs1:=(solve(localDE1,diff(F(t),[t$M])));
    eqs2:=(solve(localDE2,diff(G(t),[t$N])));
    ansatz:=add(A[u]*(diff(F(t)+G(t),[t$u])),u=0..k);
        for v from k-M to 0 by -1 do
            ansatz:=expand(subs(diff(F(t),[t$(M+v)])=diff(eqs1,[t$v]),ansatz));
        od;
    ansatz:=collect(ansatz,vars);
        for l from k-N to 0 by -1 do
            ansatz:=expand(subs(diff(G(t),[t$(N+l)])=diff(eqs2,[t$l]),ansatz));
        od;
    ansatz:=collect(ansatz,vars);
        for p from k to 0 by -1 do
            ansatz:=subs(diff(F(t),[t$p])=X^(p+1),ansatz);
            ansatz:=subs(diff(G(t),[t$p])=Y^(p+1),ansatz);
        od;
    coeff1:={seq(coeff(ansatz,X,i),i=1..k+1)};
    coeff2:={seq(coeff(ansatz,Y,i),i=1..k+1)};
    coeffs:=coeff1 union coeff2;
    sol:=solve(coeffs,var);
    ANSATZ:=subs(sol,ANSATZ);
    rule:=A(i::anything)=1;
    ANSATZ:=numer(normal(applyrule(rule,ANSATZ)));
        if not ANSATZ=0 then 
            print("The sum of the solutions of the 2 given differential equations satisfies the trigonometric holonomic differential equation:");
            return collect(numer(normal(combine(ANSATZ))),diff,factor)=0;
        fi;
od;
nosolution;
end proc: #end SumDE(48)

# The procedure SumTHDE does the same thing as the previous, but restrict itself only on trigonometric holonomic inputs. 
# Its first check is the input differential equations are trigonometric holonomic or not.
# However the output can be trigonometric holonomic or not, depending whether the both DE inputs are in the same TH(w) or not.

SumTHDE:=proc(DE1,DE2,Ft)
local F,t,localde1,localde2,varde1,varde2,var1,var2,w1,w2,localde;
F:=op(0,Ft);
t:=op(1,Ft);
localde1:=DE1;
localde2:=DE2;
varde1:=DEvariable(localde1);
varde2:=DEvariable(localde2);
if (varde1=DeOfOrderZero or varde2=DeOfOrderZero) then
  return("Each of both differential equations muss be at least of order 1");
fi;
if varde1=Error then 
    return Diffvariable(localde1);
fi;
if varde2=Error then 
   return Diffvariable(localde2);
fi;
if not type(localde1,TH_DE(varde1)) then 
 return THDEtest(localde1,varde1);
fi;
if not type(localde2,TH_DE(varde2)) then 
 return THDEtest(localde2,varde2);
fi;
var1:=op(1,(op(1,varde1)));
var2:=op(1,(op(1,varde2)));
w1:= THDEtest(localde1,varde1);
w2:= THDEtest(localde2,varde2);
if type(w1/w2,integer) or type(w1/w2,fraction) then
    if not var1=var2 then
        print("The solutions of",DE1,"and",DE2,"
        are of different arguments, namely",var1,"and",var2);
        print("But considering that those functions are both of variable",t,"then we will have");
        localde1:=subs(var1=t,localde1);
        localde2:=subs(var2=t,localde2);
        localde:=(SumDE(localde1,localde2,F(t)));
        print(localde);
        return("which is a trigonometric holonomic differential equation of TH(",THDEtest(localde,F(t)),")");
    fi;
    localde:=SumDE(localde1,localde2,F(t));
    print(localde);
    return("which is a trigonometric holonomic differential equation of TH(",THDEtest(localde,F(t)),")");
fi;
print(localde1," is in TH(",w1,"), \n however",localde2,"is in TH(",w2,"), \n i.e. the coefficients of each of both differential equations are not commensurable one to another");

  if not var1=var2 then
        print("The solutions of",DE1,"and",DE2,"
        are of different arguments, namely",var1,"and",var2);
        print("But considering that those functions are both of variable",t,"then we will have");
        localde1:=subs(var1=t,localde1);
        localde2:=subs(var2=t,localde2);
        localde:=(SumDE(localde1,localde2,F(t)));
        print(localde);
        if type(localde,TH_DE(F(t))) then 
         return("which is nevertheless a trigonometric holonomic differential equation of TH(",THDEtest        (localde,F(t)),")");
        fi;
            return("which is not a trigonometric holonomic differential equation");
        fi;
localde:=SumDE(localde1,localde2,F(t));
print(localde);
if type(localde,TH_DE(F(t))) then 
print("which is nevertheless a trigonometric holonomic differential equation of TH(",THDEtest(localde,F(t)),")");
else
print("which is not a trigonometric holonomic differential equation");
fi;
end proc: #end SumTHDE (65)

# The following procedure determines a differential equation satisfied by the product of the solutions of 2  differential equations DE1 and DE2.
# Note that this procedure does not care whether the input differential equations are trigonometric holonomic or not.
# But the corresponding functions muss be of the same arguments. ( t or x for example)
# 
ProductDE:=proc(DE1,DE2,Ft)
local f,t,localDE1,localDE2,M,N,k,vars,var,A,r,s,u,ANSATZ,eqs1,eqs2,ansatz,v,l,p,coeff,sol,rule ;
f:=op(0,Ft);
t:=op(1,Ft);
localDE1:=DE1;
localDE2:=DE2;
M:=PDEtools[difforder](localDE1,t);
N:=PDEtools[difforder](localDE2,t);
for k from max(M,N) to M*N do
    vars:=op(add(diff(F(t)*G(t),[t$i]),i=0..k));
    var:={seq(A[r],r=0..k)};
    ANSATZ:=sum(A[s]*diff(f(t),[t$s]),s=0..k);
    localDE1:=subs(op(0,DEvariable(localDE1))=F,localDE1);
    localDE2:=subs(op(0,DEvariable(localDE2))=G,localDE2);
    eqs1:=(solve(localDE1,diff(F(t),[t$M])));
    eqs2:=(solve(localDE2,diff(G(t),[t$N])));
    ansatz:=add(A[u]*(diff(F(t)*G(t),[t$u])),u=0..k);
        for v from k-M to 0 by -1 do
            ansatz:=expand(subs(diff(F(t),[t$(M+v)])=diff(eqs1,[t$v]),ansatz));
        end do;
        for l from k-N to 0 by -1 do
            ansatz:=expand(subs(diff(G(t),[t$(N+l)])=diff(eqs2,[t$l]),ansatz));
        od;
    ansatz:=numer(combine(normal(ansatz)));
    ansatz:=collect( ansatz, diff, `distributed`,factor );
        for p from k to 0 by -1 do
            ansatz:=subs(diff(F(t),[t$p])=X^(p+1),ansatz);
            ansatz:=subs(diff(G(t),[t$p])=Y^(p+1),ansatz);
        od;
    coeff:={coeffs(ansatz,[X,Y])}; 
    sol:=solve(coeff,var);
        if sol={} then
            return sol ;
        fi;
    ANSATZ:=subs(sol,ANSATZ);
    rule:=A(i::anything)=1;
    ANSATZ:=numer(normal(applyrule(rule,ANSATZ)));
        if not ANSATZ=0 then
            print("The product of the solutions of the 2 given differential equations satisfies the trigonometric holonomic differential equation:");
            return collect(numer(normal(combine(ANSATZ))),diff,factor)=0;
        fi;
od;
nosolution;
end proc:#end ProductDE (44)

# The procedure ProductTHDE does the same thing as the previous, but restrict itself only on trigonometric holonomic inputs. 
# Its first check if the input differential equations are trigonometric holonomic or not.
# However the output can be trigonometric holonomic or not, depending whether the both DE inputs are in the same TH(w) or not.
ProductTHDE:=proc(DE1,DE2,Ft)
local localde1,localde2,varde1,varde2,F,t,var1,var2,w1,w2,localde;
F:=op(0,Ft);
t:=op(1,Ft);
localde1:=DE1;
localde2:=DE2;
varde1:=DEvariable(localde1);
varde2:=DEvariable(localde2);

if (varde1=DeOfOrderZero or varde2=DeOfOrderZero0) then
  return("Each of both differential equations muss be at least of order 1");
fi;
if varde1=Error then 
    return Diffvariable(localde1);
fi;
if varde2=Error then 
   return Diffvariable(localde2);
fi;
if not type(localde1,TH_DE(varde1)) then 
 return THDEtest(localde1,varde1);
fi;
if not type(localde2,TH_DE(varde2)) then 
 return THDEtest(localde2,varde2);
fi;
var1:=op(1,(op(1,varde1)));
var2:=op(1,(op(1,varde2)));
w1:= THDEtest(localde1,varde1);
w2:= THDEtest(localde2,varde2);
if type(w1/w2,integer) or type(w1/w2,fraction) then
    if not var1=var2 then
        print("The solutions of",DE1,"and",DE2," \n are of different arguments, namely",var1,"and",var2);
        print("But considering that those functions are both of variable",t,"then we will have"); \n localde1:=subs(var1=t,localde1);
        localde2:=subs(var2=t,localde2);
        localde:=(ProductDE(localde1,localde2,F(t)));
        print(localde);
        return("which is a trigonometric holonomic differential equation of TH(",THDEtest(localde,F(t)),")");
    fi;
    localde:=ProductDE(localde1,localde2,F(t));
    print(localde);
    return("which is a trigonometric holonomic differential equation of TH(",THDEtest(localde,F(t)),")");
fi;
print(localde1," is in TH(",w1,"),\n however",localde2,"is in TH(",w2,"), \n i.e. the coefficients of each of both differential equations are not commensurable one to another");
if not var1=var2 then
    print("The solutions of",DE1,"and",DE2," \n are of different arguments, namely",var1,"and",var2);
    print("But considering that those functions are both of variable",t,"then we will have");
    localde1:=subs(var1=t,localde1);
    localde2:=subs(var2=t,localde2);
    localde:=(ProductDE(localde1,localde2,F(t)));
    print(localde);
    if type(localde,TH_DE(F(t))) then 
        return("which is nevertheless a trigonometric holonomic differential equation of TH(",THDEtest(localde,F(t)),")");
    fi;
        return("which is not a trigonometric holonomic differential equation");
    fi;
localde:=ProductDE(localde1,localde2,F(t));
print(localde);
if type(localde,TH_DE(F(t))) then 
print("which is nevertheless a trigonometric holonomic differential equation of TH(",THDEtest(localde,F(t)),")");
else
print("which is not a trigonometric holonomic differential equation");
fi;
end proc:#end ProductTHDE (62)

# Part 4  The procedures of this part are used in the some of the previous one.
# The following procedure replaces the parameters depending on transcendental functions cos, sin, cosh, sinh, ln, exp by another simple parameter, in order to avoid conflict (of the command "combine" for example) during the computations. It is used in the most of procedures involving the computation of trigonometric holonomic differential equations.

dec:=proc(g,t)
local a,b,i,j,e,d,f,u,v,w,x,alpha,o,p,q,e1,e2,e3,e4,r,z,o1,p1,e5,e11,x1,w1;
f:=simplify(expand(g));
a:={op(f)};
b:=[];
for i from 1 to nops(a) do
    if nops(op(i,a))=1 then
        b:=[op(b),op(i,a)];
    else 
        b:=[op(b),op(op(i,a))];
    fi;
od;
e:=select(has,b,{cos,sin,sinh,cosh,tan,ln,exp});
e:=simplify(e);
e1:=decprod(e);
if {e1}={} then 
 return ([f,{}]) ;
fi;
e1:=select(has,e1,{cos,sin,sinh,cosh,tan,ln,exp});
e1:=simplify(e1);
e2:=decsum(e1);
e2:=decpower(e2);
e2:=select(has,e2,{cos,sin,sinh,cosh,tan,ln,exp});
u:=[];
for v from 1 to nops(e2) do
    if type(op(v,e2),whattype(sqrt(2))) then 
        if not has(op(v,e2),t) then
            u:=[op(u),op(v,e2)];
        else 
           u:=[op(u),op(op(1,[op(op(v,e2))]))];  
        fi;
    elif
        has(op(v,e2),t) and  type(op(v,e2),whattype(2+t)) then
        u:=[op(u),op(op(v,e2))];
     else 
        u:=[op(u),op(v,e2)];
    fi;
od;
e3:=u;
e3:=select(has,e3,{cos,sin,sinh,cosh,tan,ln,exp});
e11:=decsum(e3);
e11:=decpower(e11);
e11:=select(has,e11,{cos,sin,sinh,cosh,tan,ln,exp});
w1:=[];
for x1 from 1 to nops(e11) do
    if nops({op(op(x1,e11))})>1 then
        w1:=[op(w1),op(op(x1,e11))];
    else 
        w1:=[op(w1),op(x1,e11)];
    fi;
od;
e4:=w1;
e4:=select(has,e4,{cos,sin,sinh,cosh,tan,ln,exp});  
e4:=decprod(e4);
e4:=decpower(e4);
o1:={};
for p1 from 1 to nops(e4) do
    if has(op(p1,e4),t) then 
        if type(op(op(p1,e4)),whattype(2+t)) then
            o1:=[op(o1),op(op(op(p1,e4)))]; 
        else
            o1:=[op(o1),op(p1,e4)];
        fi;
    else
        o1:=[op(o1),op(p1,e4)];
    fi;
od;
e5:=o1;
e5:=select(has,e5,{cos,sin,sinh,cosh,tan,ln,exp});
d:={};
for j from 1 to nops(e5) do
    if not has(op(j,e5),t) then 
        f:=subs(op(j,e5)=alpha[j],f);
        d:={op(d),alpha[j]=op(j,e5)};
    fi;
od;
return([f,d]);    
end proc:#end dec(78)

decsum:=proc(f)
local g,r,e3,z;
e3:=f;
z:={};
for r from 1 to nops(e3) do
    if type(op(r,e3),whattype(2+t)) then
        z:={op(z),op(op(r,e3))};
    else z:={op(z),op(r,e3)};
    fi;
od;
end proc:#end decsum (11)
decprod:=proc(f)
local g,r,e3,z;
e3:=f;
z:={};
for r from 1 to nops(e3) do
    if type(op(r,e3),whattype(2*t)) then
        z:={op(z),op(op(r,e3))};
    else z:={op(z),op(r,e3)};
    fi;
od;
end proc: #end decprod (11)

decpower:=proc(f)
local g,r,e3,z;
e3:=f;
z:={};
for r from 1 to nops(e3) do
    if type(op(r,e3),whattype(sqrt(2))) then
        z:={op(z),op(op(r,e3))};
    else z:={op(z),op(r,e3)};
    fi;
od;
end proc: #end decpower(11)

# The following procedure checks if an expression  $f$ of a particular form is a trigonometric polynomial, and returns false if it is not the case, or a number w such that $f$ is in TH(w).
easy:=proc(f,t)
local b,e,i,A,B,C,j,m,p,x,y,q,w,o,E;
b:=(f);
e:={};
for i from 1 to nops((b)) do  
    if not type(op(i,(b)),whattype(2*t)) then 
        e:={op(e),(op(i,b))} ;
    else
        e:={op(e),op(op(i,b))};
    fi;
od;
b:= [seq(select(has,op(i,(e)),t),i=1..nops(e))];
if nops(b)=1 then 
    if has(op(op(1,b))/t,t) then 
        return false;
    fi; 
        return   (op(op(1,b))/t);
fi;
A:=[seq((op(1,op(i,b)),i=1..nops(b)))];
B:=[seq(denom(op(z,A)),z=1..nops(A))];
C:=[seq(numer(op(u,A)/t),u=1..nops(A))];
for i from 1 to nops(A) do
     if has(op(i,A)/t,t) then 
         return false;
     fi;
od;
for i from 1 to nops(A) do      
    for j from i+1 to nops(A) do
        if not type(op(i,A)/op(j,A),rational) then
            return false;
        fi;
     od; 
 od;  
m:={};
    for p from 1 to nops((C)) do  
        if not type(op(p,(C)),whattype(2*t)) then 
            m:={op(m),(op(p,C))} ;
        else
            m:={op(m),op(op(p,C))};
        fi;
    od;
w:=m;
x:={};
y:={};
for q from 1 to nops(w) do ;
    if type(op(q,w),whattype(sqrt(2))) then  ;
      x:={op(x),op(q,w)}; 
    else
       y:={op(y),op(q,w)};
     fi;
  od;  
if not x={} then 
    if nops(x)=1 then 
       x:=op(x);
    else
        x:=product(op(o,x),o=1..nops(x));
    fi;
    C:=[seq(op(i,C)/x,i=1..nops(C))];
    return x*gcd(op(1,C),op(2,C))/lcm(op(B));
fi;
for i from 1 to nops(C) do      
    for j from i+1 to nops(C) do
        if not op(i,C)=op(j,C) then
            E:=[op(1,C),op(2,C)];
            return gcd(op(E))/lcm(op(B));
        fi;
     od; 
 od; 
op(1,C)/lcm(op(B));
end proc: #end easy (70)

# The following procedure checks if an expression  $f$ of a particular form is a trigonometric polynomial, and returns true or false, according to the result.
easytrig:=proc(f,t)
local b,e,i,A,B,C,j,m,p,x,y,q,w,o,E;
b:=(f);
e:={};
for i from 1 to nops((b)) do  
    if not type(op(i,(b)),whattype(2*t)) then 
        e:={op(e),(op(i,b))} ;
    else
        e:={op(e),op(op(i,b))};
    fi;
od;
b:= [seq(select(has,op(i,(e)),t),i=1..nops(e))];
if nops(b)=1 then 
    if has(op(op(1,b))/t,t) then 
        return false;
    fi; 
    return  true;
fi;
A:=[seq((op(1,op(i,b)),i=1..nops(b)))];
B:=[seq(denom(op(z,A)),z=1..nops(A))];
C:=[seq(numer(op(u,A)/t),u=1..nops(A))];
for i from 1 to nops(A) do
     if has(op(i,A)/t,t) then 
         return false;
     fi;
od;
for i from 1 to nops(A) do      
    for j from i+1 to nops(A) do
        if not type(op(i,A)/op(j,A),rational) then
            return false;
        fi;
     od; 
 od;  
m:={};
for p from 1 to nops((C)) do  
    if not type(op(p,(C)),whattype(2*t)) then 
        m:={op(m),(op(p,C))} ;
    else
        m:={op(m),op(op(p,C))};
    fi;
od;
w:=m;
x:={};
y:={};
for q from 1 to nops(w) do ;
    if type(op(q,w),whattype(sqrt(2))) then  ;
        x:={op(x),op(q,w)}; 
    else
        y:={op(y),op(q,w)};
    fi;
od;  
if not x={} then 
    if nops(x)=1 then 
        x:=op(x);
    else
        x:=product(op(o,x),o=1..nops(x));
    fi;
    return true;
fi;
for i from 1 to nops(C) do      
    for j from i+1 to nops(C) do
        if not op(i,C)=op(j,C) then
            E:=[op(1,C),op(2,C)];
            return true;
        fi;
     od; 
 od; 
true;
end proc:#end easytrig (69)

# The following procedure Evalf returns zero if the input is less than 10^(-10) otherwise its returns the same input.
Evalf:=proc(term)
local result;
result:=evalf(term);
if Im(result)<10^(-10) and Re(result)<10^(-10) then 
    0; 
elif
    Im(result)<10^(-10) then 
    Re(result); 
elif
    Re(result)<10^(-10) then Im(result);
fi; 
end:# end Evalf (12)


# The following procedure bringt every recurrence equation RE of variable c(n) into the form sum(a(i)*c(n), i=0..L), where L is the order of RE.
shiff:=proc(RE,cn)
local localre,c,n,L,dc,l,A,i,m;
if op(2,RE)=0 then 
    localre:=op(1,RE);
else
    localre:=RE;
fi;
L:=orderrecur(RE,cn);
c:=op(0,cn);
n:=op(1,cn);
dc:=decprod([op(expand(localre))]);
dc:=select(has,dc,c);
A:=[];
for i from 1 to nops(dc) do
    A:=[op(A),op(1,op(i,dc))-n];
od;
m:=min(A);
localre:=subs(n=n-m,localre);
localre:=collect(expand(localre),c,factor);
end proc: #end shiff

# The following procedure determines the recurrence operator corresponding to a given recurrence equation.
toN:=proc(RE,cn)
local localre,c,n,L,l;
if op(2,RE)=0 then 
    localre:=op(1,RE);
else
    localre:=RE;
fi;
localre:=shiff(localre,cn);
L:=orderrecur(localre,cn);
c:=op(0,cn);
n:=op(1,cn);
for l from 0 to L  do
    localre:=subs(c(n+l)=N^(l),localre);
    localre:=subs(c(n-l)=N^(l),localre);
od;
collect(localre,N);
end proc: #end toN (17)


# The following procedure determines  the  recurrence equation corresponding to a given recurrence operator. 
tore:=proc(Nexp,cn)
local localN,localre,l,c,n,L,i;
if op(2,Nexp)=0 then 
    localN:=op(1,Nexp);
else
    localN:=Nexp;
fi;
c:=op(0,cn);
n:=op(1,cn);
localN:=[op(localN)];
localre:={};
for i from 1 to nops(localN) do
   L:=degree(op(i,localN),N);
    if L=0 then 
        localre:=[op(localre),op(i,localN)*c(n)]; 
    else
        localre:=[op(localre),subs(N^L=c(n+L),op(i,localN))];
    fi;
od;
localre:=add(op(i,localre),i=1..nops(localre));
collect(localre,c,factor)=0;
end proc:#end tore(22)

TrigtoRE:=proc(DE,f,Ft,cn,a,b)
local DElocal,F,t,g,num,num1,numRule,i,j,den,den1,denRule,c,n,w,P,L,p,l,alpha,beta,Dif,RE,eq,ANSATZ; 
if op(2,DE)= 0 then
DElocal:=op(1,DE);
else DElocal:=DE;
end if;
F:=op(0,Ft);
t:=op(1,Ft);
num:=numer(DElocal);
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=denom(DElocal);
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
DElocal:=num/den;
DElocal:=combine(DElocal);
c:=op(0,cn);
n:=op(1,cn);
w:=2*Pi/(b-a);
P:=PDEtools[difforder](DE,t);
L:=[(op(map((y->y/t,map(x->op(1,x),select(has,indets(DElocal),[cos,sin]))))))];
L:=max(seq(op(i,L)/w,i=1..nops(L)));
L:=simplify(L);
if L<0 then
L:=1;
fi;
if L=-infinity then 
    L:=0;
fi;
for p from P to 0 by -1 do
    for l from 0 to L do 
        DElocal:=subs(diff(F(t),[t$p])=Z^p,DElocal);
        DElocal:=subs(cos(l*w*t)=X^l,DElocal); 
        DElocal:=subs(sin(l*w*t)=Y^l,DElocal);
    od;
od;
for p from 0 to P do 
    for l from 0 to L do
        alpha[p,l]:=coeff(coeff(coeff(DElocal,Z,p),X,l),Y,0);
        beta[p,l]:=coeff(coeff(coeff(DElocal,Z,p),Y,l),X,0);
    od; 
od;
ANSATZ:=0; 
for p from 0 to P do 
    if p=0 then 
        Dif:=F(t); 
    else 
        Dif:=diff(F(t),t$p);
    fi;
    for l from 0 to L do 
        ANSATZ:=ANSATZ+(alpha[p,l]*cos(l*w*t)+beta[p,l]*sin(l*w*t))*Dif;
    od;  
od;
RE:=add(2*alpha[p,0]*c[p](n),p=0..P)+ add(add(c[p](n+l)*(alpha[p,l]+I*beta[p,l])+c[p](n-l)*(alpha[p,l]-I*beta[p,l]),l=1..L),p=0..P) ;
for p from 0 to P do
    for l from -L to L do  
        eq :=c[p](n+l)=(2*(n+l)*Pi*I/(b-a))^p*(c(n+l)); 
        RE := subs(eq, RE) ;
    od;
od;
RE:=collect(numer(normal(RE)),c,factor) ;
RE:=subs(numRule,denRule,RE);
RE:=collect(expand(RE),c,factor);
print("and its Fourier coefficients satisfy the following homogeneous recurrence equation of order", orderrecur(RE,c(n)));
RE=0;
end: #end TtoRE;(68)

ExptoRE:=proc(DE,f,Ft,cn,a,b)
local DElocal,F,t,g,num,num1,numRule,den,den1,denRule,c,n,w,T,P,L1,L2,L,p,l,gamma,delta,RE,dif,ANSATZ,DIF,maxi,mini;
if op(2,DE)= 0 then
DElocal:=op(1,DE);
else DElocal:=DE;
end if;
F:=op(0,Ft);
t:=op(1,Ft);
num:=simplify(numer(DElocal));
num1:=dec(num,t);
num:=op(1,num1);
numRule:=op(2,num1);
den:=simplify(denom(DElocal));
den1:=dec(den,t);
den:=op(1,den1);
denRule:=op(2,den1);
DElocal:=num/den;
c:=op(0,cn);
n:=op(1,cn);
w:=2*Pi/(b-a);
T:=2*Pi/w;
P:=PDEtools[difforder](DElocal,t);
L1:=map((y->y/t*I,map(x->op(1,x),select(has,indets(DElocal),exp))));
L2:=map((y->y/t*1/I,map(x->op(1,x),select(has,indets(DElocal),exp))));
L:=L1 union L2;
L:=max(seq(op(i,L)/w,i=1..nops(L)));
if L=-infinity then 
    L:=0;
fi;
DElocal:=simplify(DElocal);
for p from P to 0 by -1 do
    for l from 0 to L do 
        DElocal:=subs(diff(F(t),[t$p])=Z^(p+1),DElocal);
        DElocal:=subs(exp(-I*l*w*t)=X^l,DElocal); 
        DElocal:=subs(exp(I*l*w*t)=Y^l,DElocal);
    od;
od;
for p from 1 to P+1 do 
    for l from 0 to L do
        delta[p-1,l]:=coeff(coeff(coeff(DElocal,Z,p),Y,l),X,0);
        DElocal:=DElocal-delta[p-1,l]*Z^p*Y^l;
        gamma[p-1,l]:=coeff(coeff(coeff(DElocal,Z,p),X,l),Y,0);
        DElocal:=DElocal-gamma[p-1,l]*Z^p*X^l;
    od; 
od;
ANSATZ:=0;
for p from 0 to P do
    for l from 0 to L do  
        ANSATZ:=ANSATZ+(gamma[p,l]*exp(-I*l*w*t)+delta[p,l]*exp(I*l*w*t))*diff(F(t),[t$p]);
    od; 
od;  
RE:=add(add(delta[p,l]*(2*(n-l)*Pi*I/T)^p*(c(n-l))+gamma[p,l]*(2*(n+l)*Pi*I/T)^p*(c(n+l)),l=0..L),p=0..P);
RE:=(collect(numer(normal(simplify(RE))),c,factor)) ;
RE:=subs(numRule,denRule,RE);
RE:=collect(expand(RE),c,factor);
print("and its Fourier coefficients satisfy the following homogeneous recurrence equation of order", orderrecur(RE,c(n)));
RE=0; 
end: #end EtoRE;(58)

RightFactorE:=proc(ope,n)
local localope,localre,localde,localsol,ff,d;
localope:=ope;
localre:=tore(localope,c(n));
print("The corresponding holonomic recurence equation RE to the previous holonomic recurrence operator is");
print(localre);
localde:=ReverseExpTHDEtoRE(localre,c(n),0,2*Pi);
print("The conversion of the RE into a boundary value problem with the initial values F(0)=F(2*Pi) is");
print(localde);
localsol:=op(2,dsolve(localde));
if type(localsol,whattype(2+t)) then 
      localsol:=remove(has,localsol,Int);
      if type(localsol,whattype(2+t)) then 
          ff:=op(1,localsol);
      else
          ff:=localsol;
      fi;
      print("one of whose solution is f(t)=",ff, "and");
      localde:=ExpTHDE(ff,F(t),1);
      localre:=ExptoRE(localde,ff,F(t),c(n),0,2*Pi);
      print(localre);
      print("from which we deduce the folowing right hand factor of order", orderrecur(localre,c(n)));
      print(toN(localre,c(n)));
elif not has(localsol,Int) then
      ff:=localsol;
      print("one of whose solution is f(t)=",ff, "and");
      if type(ff,RATTRIG(t)) then
          if simplify(expand(subs(t=t+Pi,ff)-ff))=0 then
             localde:=ExpTHDE(ff,F(t),2);
             localre:=ExptoRE(localde,ff,F(t),c(n),0,Pi);
          else
             localde:=ExpTHDE(ff,F(t),1); 
             localre:=ExptoRE(localde,ff,F(t),c(n),0,2*Pi);
          fi;
          print(localre);
       else
       localde:=ExpTHDE(ff,F(t),1); 
       localre:=ExptoRE(localde,ff,F(t),c(n),0,2*Pi);
       print(localre);
        fi;
       print("from which we deduce the folowing right hand factor of order", orderrecur(localre,c(n)));
       toN(localre,c(n));
     else
return("The seach for a right hand factor cannot be completed, since \n the previous boundary value problem cannot be solved");
fi;
end proc: #end RightFactorE (46);

RightFactorT:=proc(ope,n)
local localope,localre,localde,localsol,ff,d;
localope:=ope;
localre:=tore(localope,c(n));
print("The corresponding holonomic recurence equation RE to the previous holonomic recurrence operator is");
print(localre);
localde:=ReverseTHDEtoRE(localre,c(n),0,2*Pi);
print("The conversion of the RE into a boundary value problem with the initial values F(0)=F(2*Pi) is");
print(localde);
localsol:=op(2,dsolve(localde));
if type(localsol,whattype(2+t)) then 
      localsol:=remove(has,localsol,Int);
      if type(localsol,whattype(2+t)) then 
          ff:=op(1,localsol);
      else
          ff:=localsol;
      fi;
      print("one of whose solution is f(t)=",ff, "and");
      if type(ff,RATTRIG(t)) then
          if simplify(expand(subs(t=t+Pi,ff)-ff))=0 then
             localde:=THDE(ff,F(t),2);
             localre:=TrigtoRE(localde,ff,F(t),c(n),0,Pi);
          else
             localde:=THDE(ff,F(t),1); 
             localre:=TrigtoRE(localde,ff,F(t),c(n),0,2*Pi);
          fi;
          print(localre);
       else
       localde:=THDE(ff,F(t),1); 
       localre:=TrigtoRE(localde,ff,F(t),c(n),0,2*Pi);
       print(localre);
        fi;
       print("from which we deduce the folowing right hand factor of order", orderrecur(localre,c(n)));
       print(toN(localre,c(n)));
elif not has(localsol,Int) then
      ff:=localsol;
      print("one of whose solution is f(t)=",ff, "and");
      if type(ff,RATTRIG(t)) then
          if simplify(expand(subs(t=t+Pi,ff)-ff))=0 then
             localde:=THDE(ff,F(t),2);
             localre:=TrigtoRE(localde,ff,F(t),c(n),0,Pi);
          else
             localde:=THDE(ff,F(t),1); 
             localre:=TrigtoRE(localde,ff,F(t),c(n),0,2*Pi);
          fi;
          print(localre);
       else
       localde:=THDE(ff,F(t),1); 
       localre:=TrigtoRE(localde,ff,F(t),c(n),0,2*Pi);
       print(localre);
        fi;
       print("from which we deduce the folowing right hand factor of order", orderrecur(localre,c(n)));
       toN(localre,c(n));
     else
return("The seach for a right hand factor cannot be completed, since \n the previous boundary value problem cannot be solved");
fi;
end proc:# end RightFactorT (57);
# We have here the total number of the lines of the different above procedures.
#199+193+63+63+66+73+44+45+7+10+47+35+57+35+12+44+69+72+7+25+28+9+54+
#53+20+81+46+47+74+33+62+11+9+6+20+69+11+43+11+20+17+22+44+45+48+65+44#+62+78+11+11+11+70+69+12+68+58+46+57;
#around 2711 lines.

